@@ -16,14 +16,12 @@
 */
 
 #include <iostream>
-#include <string>
 #include <ignition/transport.hh>
 #include "msgs/stringmsg.pb.h"
 
 //////////////////////////////////////////////////
 /// \brief Service response callback.
-void responseCb(const std::string &_topic, const example::msgs::StringMsg &_rep,
-  bool _result)
+void responseCb(const example::msgs::StringMsg &_rep, const bool _result)
 {
   if (_result)
     std::cout << "Response: [" << _rep.data() << "]" << std::endl;
@@ -41,10 +39,11 @@ int main(int argc, char **argv)
   example::msgs::StringMsg req;
   req.set_data("HELLO");
 
+  std::cout << "Press <CTRL-C> to exit" << std::endl;
+
   // Request the "/echo" service.
   node.Request("/echo", req, responseCb);
 
-  // Wait for the response.
-  std::cout << "Press <ENTER> to exit" << std::endl;
-  getchar();
+  // Zzzzzz.
+  ignition::transport::waitForShutdown();
 }
