@@ -15,14 +15,14 @@
  *
 */
 
-#include <cstdio>
+#include <iostream>
 #include <string>
 #include <ignition/transport.hh>
 #include "msgs/stringmsg.pb.h"
 
 //////////////////////////////////////////////////
 /// \brief Provide an "echo" service.
-void srvEcho(const std::string &_topic, const example::msgs::StringMsg &_req,
+void srvEcho(const example::msgs::StringMsg &_req,
   example::msgs::StringMsg &_rep, bool &_result)
 {
   // Set the response's content.
@@ -51,6 +51,6 @@ int main(int argc, char **argv)
     return -1;
   }
 
-  // Wait for requests.
-  getchar();
+  // Zzzzzz.
+  ignition::transport::waitForShutdown();
 }
