@@ -15,7 +15,6 @@
  *
 */
 
-#include <cstdio>
 #include <iostream>
 #include <string>
 #include <ignition/transport.hh>
@@ -23,10 +22,9 @@
 
 //////////////////////////////////////////////////
 /// \brief Function called each time a topic update is received.
-void cb(const std::string &_topic, const example::msgs::StringMsg &_msg)
+void cb(const example::msgs::StringMsg &_msg)
 {
-  std::cout << "Topic:" << _topic << "\n"
-            << "Msg:  " << _msg.data() << "\n\n";
+  std::cout << "Msg: " << _msg.data() << std::endl << std::endl;
 }
 
 //////////////////////////////////////////////////
@@ -43,8 +41,7 @@ int main(int argc, char **argv)
   }
 
   // Zzzzzz.
-  std::cout << "Press <ENTER> to exit" << std::endl;
-  getchar();
+  ignition::transport::waitForShutdown();
 
   return 0;
 }
