@@ -15,9 +15,6 @@
  *
 */
 
-#ifdef _MSC_VER
-# pragma warning(push, 0)
-#endif
 #ifdef _WIN32
   // For socket(), connect(), send(), and recv().
   #include <Winsock2.h>
@@ -43,20 +40,32 @@
   using raw_type = void;
 #endif
 
+#ifdef _MSC_VER
+  #pragma warning(push, 0)
+#endif
 #include <zmq.hpp>
+#ifdef _WIN32
+  #pragma warning(pop)
+  // Suppress "decorated name length exceed" warning in STL.
+  #pragma warning(disable: 4503)
+  // Suppress "depreted API warnings" in WINSOCK.
+  #pragma warning(disable: 4996)
+#endif
+
 #include <chrono>
 #include <iostream>
 #include <mutex>
 #include <string>
 #include <vector>
-#ifdef _MSC_VER
-# pragma warning(pop)
-#endif
+
+#include "ignition/transport/AdvertiseOptions.hh"
 #include "ignition/transport/Discovery.hh"
 #include "ignition/transport/DiscoveryPrivate.hh"
+#include "ignition/transport/Helpers.hh"
 #include "ignition/transport/NetUtils.hh"
 #include "ignition/transport/Packet.hh"
 #include "ignition/transport/Publisher.hh"
+#include "ignition/transport/TopicStorage.hh"
 #include "ignition/transport/TransportTypes.hh"
 
 using namespace ignition;
@@ -85,8 +94,14 @@ Discovery::Discovery(const std::string &_pUuid, bool _verbose)
   // Get this host IP address.
   this->dataPtr->hostAddr = determineHost();
 
-  // Get the list of network interfaces in this host.
-  this->dataPtr->hostInterfaces = determineInterfaces();
+  std::string ignIp;
+  if (env("IGN_IP", ignIp) && !ignIp.empty())
+    this->dataPtr->hostInterfaces = {ignIp};
+  else
+  {
+    // Get the list of network interfaces in this host.
+    this->dataPtr->hostInterfaces = determineInterfaces();
+  }
 
 #ifdef _WIN32
   if (!initialized)
@@ -109,42 +124,20 @@ Discovery::Discovery(const std::string &_pUuid, bool _verbose)
 
   for (const auto &netIface : this->dataPtr->hostInterfaces)
   {
-    // Make a new socket for sending discovery information.
-    int sock = socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP);
-    if (sock < 0)
-    {
-      std::cerr << "Socket creation failed." << std::endl;
-      return;
-    }
+    auto succeed = this->RegisterNetIface(netIface);
 
-    // Socket option: IP_MULTICAST_IF.
-    // This socket option needs to be applied to each socket used to send data.
-    // This option selects the source interface for outgoing messages.
-    struct in_addr ifAddr;
-    ifAddr.s_addr = inet_addr(netIface.c_str());
-    if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF,
-      reinterpret_cast<const char*>(&ifAddr), sizeof(ifAddr)) != 0)
+    // If the IP address that we're selecting as the main IP address of the host
+    // is invalid, we change it to 127.0.0.1 . This is probably because IGN_IP
+    // is set to a wrong value.
+    if (netIface == this->dataPtr->hostAddr && !succeed)
     {
-      std::cerr << "Error setting socket option (IP_MULTICAST_IF)."
-                << std::endl;
-      return;
-    }
-
-    this->dataPtr->sockets.push_back(sock);
-
-    // Join the multicast group. We have to do it for each network interface but
-    // we can do it on the same socket. We will use the socket at position 0 for
-    // receiving multicast information.
-    struct ip_mreq group;
-    group.imr_multiaddr.s_addr =
-      inet_addr(this->dataPtr->MulticastGroup.c_str());
-    group.imr_interface.s_addr = inet_addr(netIface.c_str());
-    if (setsockopt(this->dataPtr->sockets.at(0), IPPROTO_IP, IP_ADD_MEMBERSHIP,
-      reinterpret_cast<const char*>(&group), sizeof(group)) != 0)
-    {
-      std::cerr << "Error setting socket option (IP_ADD_MEMBERSHIP)."
-                << std::endl;
-      return;
+      this->RegisterNetIface("127.0.0.1");
+      std::cerr << "Did you set the environment variable IGN_IP with a correct "
+                << "IP address? " << std::endl
+                << "  [" << netIface << "] seems an invalid local IP address."
+                << std::endl
+                << "  Using 127.0.0.1 as hostname." << std::endl;
+      this->dataPtr->hostAddr = "127.0.0.1";
     }
   }
 
@@ -177,7 +170,8 @@ Discovery::Discovery(const std::string &_pUuid, bool _verbose)
   memset(&localAddr, 0, sizeof(localAddr));
   localAddr.sin_family = AF_INET;
   localAddr.sin_addr.s_addr = htonl(INADDR_ANY);
-  localAddr.sin_port = htons(this->dataPtr->DiscoveryPort);
+  localAddr.sin_port =
+      htons(static_cast<u_short>(this->dataPtr->DiscoveryPort));
 
   if (bind(this->dataPtr->sockets.at(0),
     reinterpret_cast<sockaddr *>(&localAddr), sizeof(sockaddr_in)) < 0)
@@ -191,7 +185,8 @@ Discovery::Discovery(const std::string &_pUuid, bool _verbose)
   this->dataPtr->mcastAddr.sin_family = AF_INET;
   this->dataPtr->mcastAddr.sin_addr.s_addr =
     inet_addr(this->dataPtr->MulticastGroup.c_str());
-  this->dataPtr->mcastAddr.sin_port = htons(this->dataPtr->DiscoveryPort);
+  this->dataPtr->mcastAddr.sin_port =
+      htons(static_cast<u_short>(this->dataPtr->DiscoveryPort));
 
   if (this->dataPtr->verbose)
     this->PrintCurrentState();
@@ -220,7 +215,8 @@ Discovery::~Discovery()
   if (this->dataPtr->threadActivity.joinable())
     this->dataPtr->threadActivity.join();
 #else
-  while (true)
+  bool exitLoop = false;
+  while (!exitLoop)
   {
     std::lock_guard<std::recursive_mutex> lock(this->dataPtr->exitMutex);
     {
@@ -228,7 +224,7 @@ Discovery::~Discovery()
           this->dataPtr->threadHeartbeatExiting &&
           this->dataPtr->threadActivityExiting)
       {
-        break;
+        exitLoop = true;
       }
     }
     std::this_thread::sleep_for(std::chrono::milliseconds(50));
@@ -238,7 +234,7 @@ Discovery::~Discovery()
   // Broadcast a BYE message to trigger the remote cancellation of
   // all our advertised topics.
   this->SendMsg(ByeType,
-    Publisher("", "", this->dataPtr->pUuid, "", Scope_t::All));
+    Publisher("", "", this->dataPtr->pUuid, "", Scope_t::ALL));
   std::this_thread::sleep_for(std::chrono::milliseconds(100));
 
   // Close sockets.
@@ -297,7 +293,7 @@ bool Discovery::AdvertiseMsg(const MessagePublisher &_publisher)
   this->dataPtr->infoMsg.AddPublisher(_publisher);
 
   // Only advertise a message outside this process if the scope is not 'Process'
-  if (_publisher.Scope() != Scope_t::Process)
+  if (_publisher.Scope() != Scope_t::PROCESS)
     this->SendMsg(AdvType, _publisher);
 
   return true;
@@ -315,7 +311,7 @@ bool Discovery::AdvertiseSrv(const ServicePublisher &_publisher)
   this->dataPtr->infoSrv.AddPublisher(_publisher);
 
   // Only advertise a message outside this process if the scope is not 'Process'
-  if (_publisher.Scope() != Scope_t::Process)
+  if (_publisher.Scope() != Scope_t::PROCESS)
     this->SendMsg(AdvSrvType, _publisher);
 
   return true;
@@ -332,7 +328,7 @@ bool Discovery::UnadvertiseMsg(const std::string &_topic,
 
   MessagePublisher inf;
   // Don't do anything if the topic is not advertised by any of my nodes.
-  if (!this->dataPtr->infoMsg.GetPublisher(_topic, this->dataPtr->pUuid, _nUuid,
+  if (!this->dataPtr->infoMsg.Publisher(_topic, this->dataPtr->pUuid, _nUuid,
     inf))
   {
     return true;
@@ -344,7 +340,7 @@ bool Discovery::UnadvertiseMsg(const std::string &_topic,
 
   // Only unadvertise a message outside this process if the scope
   // is not 'Process'.
-  if (inf.Scope() != Scope_t::Process)
+  if (inf.Scope() != Scope_t::PROCESS)
     this->SendMsg(UnadvType, inf);
 
   return true;
@@ -361,7 +357,7 @@ bool Discovery::UnadvertiseSrv(const std::string &_topic,
 
   ServicePublisher inf;
   // Don't do anything if the topic is not advertised by any of my nodes.
-  if (!this->dataPtr->infoSrv.GetPublisher(_topic, this->dataPtr->pUuid, _nUuid,
+  if (!this->dataPtr->infoSrv.Publisher(_topic, this->dataPtr->pUuid, _nUuid,
     inf))
   {
     return true;
@@ -373,7 +369,7 @@ bool Discovery::UnadvertiseSrv(const std::string &_topic,
 
   // Only unadvertise a message outside this process if the scope
   // is not 'Process'.
-  if (inf.Scope() != Scope_t::Process)
+  if (inf.Scope() != Scope_t::PROCESS)
     this->SendMsg(UnadvSrvType, inf);
 
   return true;
@@ -382,38 +378,42 @@ bool Discovery::UnadvertiseSrv(const std::string &_topic,
 //////////////////////////////////////////////////
 bool Discovery::DiscoverMsg(const std::string &_topic)
 {
-  std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+  MsgDiscoveryCallback cb;
+  bool found;
+  MsgAddresses_M addresses;
 
-  if (!this->dataPtr->enabled)
-    return false;
-
-  MsgDiscoveryCallback cb = this->dataPtr->connectionCb;
-  MessagePublisher pub;
-
-  pub.Topic(_topic);
-  pub.PUuid(this->dataPtr->pUuid);
-  pub.Scope(Scope_t::All);
-
-  // Broadcast a discovery request for this service call.
-  this->SendMsg(SubType, pub);
-
-  // I already have information about this topic.
-  if (this->dataPtr->infoMsg.HasTopic(_topic))
   {
-    MsgAddresses_M addresses;
-    if (this->dataPtr->infoMsg.GetPublishers(_topic, addresses))
+    std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+
+    if (!this->dataPtr->enabled)
+      return false;
+
+    cb = this->dataPtr->connectionCb;
+
+    MessagePublisher pub;
+    pub.SetTopic(_topic);
+    pub.SetPUuid(this->dataPtr->pUuid);
+    pub.SetScope(Scope_t::ALL);
+
+    // Broadcast a discovery request for this service call.
+    this->SendMsg(SubType, pub);
+
+    found = this->dataPtr->infoMsg.Publishers(_topic, addresses);
+  }
+
+  if (found)
+  {
+    // I already have information about this topic.
+    for (auto &proc : addresses)
     {
-      for (auto &proc : addresses)
+      for (auto &node : proc.second)
       {
-        for (auto &node : proc.second)
+        if (cb)
         {
-          if (cb)
-          {
-            // Execute the user's callback for a service request. Notice
-            // that we only execute one callback for preventing receive multiple
-            // service responses for a single request.
-            cb(node);
-          }
+          // Execute the user's callback for a service request. Notice
+          // that we only execute one callback for preventing receive multiple
+          // service responses for a single request.
+          cb(node);
         }
       }
     }
@@ -425,38 +425,42 @@ bool Discovery::DiscoverMsg(const std::string &_topic)
 //////////////////////////////////////////////////
 bool Discovery::DiscoverSrv(const std::string &_topic)
 {
-  std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+  SrvDiscoveryCallback cb;
+  SrvAddresses_M addresses;
+  bool found;
 
-  if (!this->dataPtr->enabled)
-    return false;
-
-  SrvDiscoveryCallback cb = this->dataPtr->connectionSrvCb;
-  ServicePublisher pub;
-
-  pub.Topic(_topic);
-  pub.PUuid(this->dataPtr->pUuid);
-  pub.Scope(Scope_t::All);
-
-  // Broadcast a discovery request for this service call.
-  this->SendMsg(SubSrvType, pub);
-
-  // I already have information about this topic.
-  if (this->dataPtr->infoSrv.HasTopic(_topic))
   {
-    SrvAddresses_M addresses;
-    if (this->dataPtr->infoSrv.GetPublishers(_topic, addresses))
+    std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+
+    if (!this->dataPtr->enabled)
+      return false;
+
+    cb = this->dataPtr->connectionSrvCb;
+
+    ServicePublisher pub;
+    pub.SetTopic(_topic);
+    pub.SetPUuid(this->dataPtr->pUuid);
+    pub.SetScope(Scope_t::ALL);
+
+    // Broadcast a discovery request for this service call.
+    this->SendMsg(SubSrvType, pub);
+
+    found = this->dataPtr->infoSrv.Publishers(_topic, addresses);
+  }
+
+  if (found)
+  {
+    // I already have information about this service.
+    for (auto &proc : addresses)
     {
-      for (auto &proc : addresses)
+      for (auto &node : proc.second)
       {
-        for (auto &node : proc.second)
+        if (cb)
         {
-          if (cb)
-          {
-            // Execute the user's callback for a service request. Notice
-            // that we only execute one callback for preventing receive multiple
-            // service responses for a single request.
-            cb(node);
-          }
+          // Execute the user's callback for a service request. Notice
+          // that we only execute one callback for preventing receive multiple
+          // service responses for a single request.
+          cb(node);
         }
       }
     }
@@ -465,12 +469,26 @@ bool Discovery::DiscoverSrv(const std::string &_topic)
   return true;
 }
 
+//////////////////////////////////////////////////
+const TopicStorage<MessagePublisher> &Discovery::DiscoveryMsgInfo() const
+{
+  std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+  return this->dataPtr->infoMsg;
+}
+
+//////////////////////////////////////////////////
+const TopicStorage<ServicePublisher> &Discovery::DiscoverySrvInfo() const
+{
+  std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+  return this->dataPtr->infoSrv;
+}
+
 //////////////////////////////////////////////////
 bool Discovery::MsgPublishers(const std::string &_topic,
                               MsgAddresses_M &_publishers)
 {
   std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
-  return this->dataPtr->infoMsg.GetPublishers(_topic, _publishers);
+  return this->dataPtr->infoMsg.Publishers(_topic, _publishers);
 }
 
 //////////////////////////////////////////////////
@@ -478,7 +496,7 @@ bool Discovery::SrvPublishers(const std::string &_topic,
                               SrvAddresses_M &_publishers)
 {
   std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
-  return this->dataPtr->infoSrv.GetPublishers(_topic, _publishers);
+  return this->dataPtr->infoSrv.Publishers(_topic, _publishers);
 }
 
 //////////////////////////////////////////////////
@@ -517,28 +535,28 @@ unsigned int Discovery::SilenceInterval() const
 }
 
 //////////////////////////////////////////////////
-void Discovery::ActivityInterval(const unsigned int _ms)
+void Discovery::SetActivityInterval(const unsigned int _ms)
 {
   std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
   this->dataPtr->activityInterval = _ms;
 }
 
 //////////////////////////////////////////////////
-void Discovery::HeartbeatInterval(const unsigned int _ms)
+void Discovery::SetHeartbeatInterval(const unsigned int _ms)
 {
   std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
   this->dataPtr->heartbeatInterval = _ms;
 }
 
 //////////////////////////////////////////////////
-void Discovery::AdvertiseInterval(const unsigned int _ms)
+void Discovery::SetAdvertiseInterval(const unsigned int _ms)
 {
   std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
   this->dataPtr->advertiseInterval = _ms;
 }
 
 //////////////////////////////////////////////////
-void Discovery::SilenceInterval(const unsigned int _ms)
+void Discovery::SetSilenceInterval(const unsigned int _ms)
 {
   std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
   this->dataPtr->silenceInterval = _ms;
@@ -575,7 +593,8 @@ void Discovery::DisconnectionsSrvCb(const SrvDiscoveryCallback &_cb)
 //////////////////////////////////////////////////
 void Discovery::RunActivityTask()
 {
-  while (true)
+  bool timeToExit = false;
+  while (!timeToExit)
   {
     this->dataPtr->mutex.lock();
 
@@ -598,8 +617,8 @@ void Discovery::RunActivityTask()
         // that a remote node is gone, even if we were not interested in its
         // topics.
         MessagePublisher publisher;
-        publisher.PUuid(it->first);
-        publisher.Scope(Scope_t::All);
+        publisher.SetPUuid(it->first);
+        publisher.SetScope(Scope_t::ALL);
         this->dataPtr->disconnectionCb(publisher);
 
         // Remove the activity entry.
@@ -623,7 +642,7 @@ void Discovery::RunActivityTask()
     {
       std::lock_guard<std::recursive_mutex> lock(this->dataPtr->exitMutex);
       if (this->dataPtr->exit)
-        break;
+        timeToExit = true;
     }
   }
 #ifdef _WIN32
@@ -635,18 +654,19 @@ void Discovery::RunActivityTask()
 //////////////////////////////////////////////////
 void Discovery::RunHeartbeatTask()
 {
-  while (true)
+  bool timeToExit = false;
+  while (!timeToExit)
   {
     {
       std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
 
       std::string pUuid = this->dataPtr->pUuid;
-      Publisher pub("", "", this->dataPtr->pUuid, "", Scope_t::All);
+      Publisher pub("", "", this->dataPtr->pUuid, "", Scope_t::ALL);
       this->SendMsg(HeartbeatType, pub);
 
       // Re-advertise topics that are advertised inside this process.
       std::map<std::string, std::vector<MessagePublisher>> msgNodes;
-      this->dataPtr->infoMsg.GetPublishersByProc(pUuid, msgNodes);
+      this->dataPtr->infoMsg.PublishersByProc(pUuid, msgNodes);
       for (const auto &topic : msgNodes)
       {
         for (const auto &node : topic.second)
@@ -655,7 +675,7 @@ void Discovery::RunHeartbeatTask()
 
       // Re-advertise services that are advertised inside this process.
       std::map<std::string, std::vector<ServicePublisher>> srvNodes;
-      this->dataPtr->infoSrv.GetPublishersByProc(pUuid, srvNodes);
+      this->dataPtr->infoSrv.PublishersByProc(pUuid, srvNodes);
       for (const auto &topic : srvNodes)
       {
         for (const auto &node : topic.second)
@@ -672,11 +692,29 @@ void Discovery::RunHeartbeatTask()
       std::chrono::milliseconds(this->dataPtr->heartbeatInterval));
 #endif
 
+    {
+      std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+
+      if (!this->dataPtr->initialized)
+      {
+        ++this->dataPtr->numHeartbeatsUninitialized;
+        if (this->dataPtr->numHeartbeatsUninitialized == 2)
+        {
+          // We consider the discovery initialized after two cycles of
+          // heartbeats sent.
+          this->dataPtr->initialized = true;
+
+          // Notify anyone waiting for the initialization phase to finish.
+          this->dataPtr->initializedCv.notify_all();
+        }
+      }
+    }
+
     // Is it time to exit?
     {
       std::lock_guard<std::recursive_mutex> lock(this->dataPtr->exitMutex);
       if (this->dataPtr->exit)
-        break;
+        timeToExit = true;
     }
   }
 #ifdef _WIN32
@@ -688,15 +726,23 @@ void Discovery::RunHeartbeatTask()
 //////////////////////////////////////////////////
 void Discovery::RunReceptionTask()
 {
-  while (true)
+  bool timeToExit = false;
+  while (!timeToExit)
   {
     // Poll socket for a reply, with timeout.
     zmq::pollitem_t items[] =
     {
       {0, this->dataPtr->sockets.at(0), ZMQ_POLLIN, 0},
     };
-    zmq::poll(&items[0], sizeof(items) / sizeof(items[0]),
-      this->dataPtr->Timeout);
+    try
+    {
+      zmq::poll(&items[0], sizeof(items) / sizeof(items[0]),
+        this->dataPtr->Timeout);
+    }
+    catch(...)
+    {
+      continue;
+    }
 
     //  If we got a reply, process it.
     if (items[0].revents & ZMQ_POLLIN)
@@ -711,7 +757,7 @@ void Discovery::RunReceptionTask()
     {
       std::lock_guard<std::recursive_mutex> lock(this->dataPtr->exitMutex);
       if (this->dataPtr->exit)
-        break;
+        timeToExit = true;
     }
   }
 #ifdef _WIN32
@@ -755,8 +801,6 @@ void Discovery::DispatchDiscoveryMsg(const std::string &_fromIp, char *_msg)
   Header header;
   char *pBody = _msg;
 
-  std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
-
   // Create the header from the raw bytes.
   header.Unpack(_msg);
   pBody += header.HeaderLength();
@@ -771,8 +815,19 @@ void Discovery::DispatchDiscoveryMsg(const std::string &_fromIp, char *_msg)
   if (recvPUuid == this->dataPtr->pUuid)
     return;
 
-  // Update timestamp.
-  this->dataPtr->activity[recvPUuid] = std::chrono::steady_clock::now();
+  // Update timestamp and cache the callbacks.
+  MsgDiscoveryCallback connectCb;
+  MsgDiscoveryCallback disconnectCb;
+  SrvDiscoveryCallback connectSrvCb;
+  SrvDiscoveryCallback disconnectSrvCb;
+  {
+    std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+    this->dataPtr->activity[recvPUuid] = std::chrono::steady_clock::now();
+    connectCb = this->dataPtr->connectionCb;
+    disconnectCb = this->dataPtr->disconnectionCb;
+    connectSrvCb = this->dataPtr->connectionSrvCb;
+    disconnectSrvCb = this->dataPtr->disconnectionSrvCb;
+  }
 
   switch (header.Type())
   {
@@ -783,20 +838,24 @@ void Discovery::DispatchDiscoveryMsg(const std::string &_fromIp, char *_msg)
       advMsg.Unpack(pBody);
 
       // Check scope of the topic.
-      if ((advMsg.GetPublisher().Scope() == Scope_t::Process) ||
-          (advMsg.GetPublisher().Scope() == Scope_t::Host &&
+      if ((advMsg.Publisher().Scope() == Scope_t::PROCESS) ||
+          (advMsg.Publisher().Scope() == Scope_t::HOST &&
            _fromIp != this->dataPtr->hostAddr))
       {
         return;
       }
 
       // Register an advertised address for the topic.
-      bool added = this->dataPtr->infoMsg.AddPublisher(advMsg.GetPublisher());
+      bool added;
+      {
+        std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+        added = this->dataPtr->infoMsg.AddPublisher(advMsg.Publisher());
+      }
 
-      if (added && this->dataPtr->connectionCb)
+      if (added && connectCb)
       {
         // Execute the client's callback.
-        this->dataPtr->connectionCb(advMsg.GetPublisher());
+        connectCb(advMsg.Publisher());
       }
 
       break;
@@ -808,20 +867,24 @@ void Discovery::DispatchDiscoveryMsg(const std::string &_fromIp, char *_msg)
       advSrv.Unpack(pBody);
 
       // Check scope of the topic.
-      if ((advSrv.GetPublisher().Scope() == Scope_t::Process) ||
-          (advSrv.GetPublisher().Scope() == Scope_t::Host &&
+      if ((advSrv.Publisher().Scope() == Scope_t::PROCESS) ||
+          (advSrv.Publisher().Scope() == Scope_t::HOST &&
            _fromIp != this->dataPtr->hostAddr))
       {
         return;
       }
 
-      // Register an advertised address for the topic.
-      bool added = this->dataPtr->infoSrv.AddPublisher(advSrv.GetPublisher());
+      // Register an advertised address for the service.
+      bool added;
+      {
+        std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+        added = this->dataPtr->infoSrv.AddPublisher(advSrv.Publisher());
+      }
 
-      if (added && this->dataPtr->connectionSrvCb)
+      if (added && connectSrvCb)
       {
         // Execute the client's callback.
-        this->dataPtr->connectionSrvCb(advSrv.GetPublisher());
+        connectSrvCb(advSrv.Publisher());
       }
 
       break;
@@ -834,26 +897,31 @@ void Discovery::DispatchDiscoveryMsg(const std::string &_fromIp, char *_msg)
       auto recvTopic = subMsg.Topic();
 
       // Check if at least one of my nodes advertises the topic requested.
-      if (this->dataPtr->infoMsg.HasAnyPublishers(recvTopic,
-        this->dataPtr->pUuid))
+      MsgAddresses_M addresses;
       {
-        MsgAddresses_M addresses;
-        if (this->dataPtr->infoMsg.GetPublishers(recvTopic, addresses))
+        std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+        if (!this->dataPtr->infoMsg.HasAnyPublishers(recvTopic,
+              this->dataPtr->pUuid))
         {
-          for (auto nodeInfo : addresses[this->dataPtr->pUuid])
-          {
-            // Check scope of the topic.
-            if ((nodeInfo.Scope() == Scope_t::Process) ||
-                (nodeInfo.Scope() == Scope_t::Host &&
-                 _fromIp != this->dataPtr->hostAddr))
-            {
-              continue;
-            }
-
-            // Answer an ADVERTISE message.
-            this->SendMsg(AdvType, nodeInfo);
-          }
+          break;
         }
+
+        if (!this->dataPtr->infoMsg.Publishers(recvTopic, addresses))
+          break;
+      }
+
+      for (auto nodeInfo : addresses[this->dataPtr->pUuid])
+      {
+        // Check scope of the topic.
+        if ((nodeInfo.Scope() == Scope_t::PROCESS) ||
+            (nodeInfo.Scope() == Scope_t::HOST &&
+             _fromIp != this->dataPtr->hostAddr))
+        {
+          continue;
+        }
+
+        // Answer an ADVERTISE message.
+        this->SendMsg(AdvType, nodeInfo);
       }
 
       break;
@@ -865,27 +933,32 @@ void Discovery::DispatchDiscoveryMsg(const std::string &_fromIp, char *_msg)
       subMsg.Unpack(pBody);
       auto recvTopic = subMsg.Topic();
 
-      // Check if at least one of my nodes advertises the topic requested.
-      if (this->dataPtr->infoSrv.HasAnyPublishers(
-        recvTopic, this->dataPtr->pUuid))
+      // Check if at least one of my nodes advertises the service requested.
+      SrvAddresses_M addresses;
       {
-        SrvAddresses_M addresses;
-        if (this->dataPtr->infoSrv.GetPublishers(recvTopic, addresses))
+        std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+        if (!this->dataPtr->infoSrv.HasAnyPublishers(
+              recvTopic, this->dataPtr->pUuid))
         {
-          for (auto nodeInfo : addresses[this->dataPtr->pUuid])
-          {
-            // Check scope of the topic.
-            if ((nodeInfo.Scope() == Scope_t::Process) ||
-                (nodeInfo.Scope() == Scope_t::Host &&
-                 _fromIp != this->dataPtr->hostAddr))
-            {
-              continue;
-            }
-
-            // Answer an ADVERTISE message.
-            this->SendMsg(AdvSrvType, nodeInfo);
-          }
+          break;
         }
+
+        if (!this->dataPtr->infoSrv.Publishers(recvTopic, addresses))
+          break;
+      }
+
+      for (auto nodeInfo : addresses[this->dataPtr->pUuid])
+      {
+        // Check scope of the topic.
+        if ((nodeInfo.Scope() == Scope_t::PROCESS) ||
+            (nodeInfo.Scope() == Scope_t::HOST &&
+             _fromIp != this->dataPtr->hostAddr))
+        {
+          continue;
+        }
+
+        // Answer an ADVERTISE_SRV message.
+        this->SendMsg(AdvSrvType, nodeInfo);
       }
 
       break;
@@ -898,29 +971,35 @@ void Discovery::DispatchDiscoveryMsg(const std::string &_fromIp, char *_msg)
     case ByeType:
     {
       // Remove the activity entry for this publisher.
-      this->dataPtr->activity.erase(recvPUuid);
-
-      if (this->dataPtr->disconnectionCb)
       {
-        MessagePublisher msgPub;
-        msgPub.PUuid(recvPUuid);
-        msgPub.Scope(Scope_t::All);
-        // Notify the new disconnection.
-        this->dataPtr->disconnectionCb(msgPub);
+        std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+        this->dataPtr->activity.erase(recvPUuid);
       }
 
-      if (this->dataPtr->disconnectionSrvCb)
+      if (disconnectCb)
+      {
+        MessagePublisher msgPub;
+        msgPub.SetPUuid(recvPUuid);
+        msgPub.SetScope(Scope_t::ALL);
+        // Notify the new disconnection.
+        disconnectCb(msgPub);
+      }
+
+      if (disconnectSrvCb)
       {
         ServicePublisher srvPub;
-        srvPub.PUuid(recvPUuid);
-        srvPub.Scope(Scope_t::All);
+        srvPub.SetPUuid(recvPUuid);
+        srvPub.SetScope(Scope_t::ALL);
         // Notify the new disconnection.
-        this->dataPtr->disconnectionSrvCb(srvPub);
+        disconnectSrvCb(srvPub);
       }
 
       // Remove the address entry for this topic.
-      this->dataPtr->infoMsg.DelPublishersByProc(recvPUuid);
-      this->dataPtr->infoSrv.DelPublishersByProc(recvPUuid);
+      {
+        std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+        this->dataPtr->infoMsg.DelPublishersByProc(recvPUuid);
+        this->dataPtr->infoSrv.DelPublishersByProc(recvPUuid);
+      }
 
       break;
     }
@@ -931,24 +1010,25 @@ void Discovery::DispatchDiscoveryMsg(const std::string &_fromIp, char *_msg)
       advMsg.Unpack(pBody);
 
       // Check scope of the topic.
-      if ((advMsg.GetPublisher().Scope() == Scope_t::Process) ||
-          (advMsg.GetPublisher().Scope() == Scope_t::Host &&
+      if ((advMsg.Publisher().Scope() == Scope_t::PROCESS) ||
+          (advMsg.Publisher().Scope() == Scope_t::HOST &&
            _fromIp != this->dataPtr->hostAddr))
       {
         return;
       }
 
-      MsgDiscoveryCallback cb = this->dataPtr->disconnectionCb;
-
-      if (cb)
+      if (disconnectCb)
       {
         // Notify the new disconnection.
-        cb(advMsg.GetPublisher());
+        disconnectCb(advMsg.Publisher());
       }
 
       // Remove the address entry for this topic.
-      this->dataPtr->infoMsg.DelPublisherByNode(advMsg.GetPublisher().Topic(),
-        advMsg.GetPublisher().PUuid(), advMsg.GetPublisher().NUuid());
+      {
+        std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+        this->dataPtr->infoMsg.DelPublisherByNode(advMsg.Publisher().Topic(),
+          advMsg.Publisher().PUuid(), advMsg.Publisher().NUuid());
+      }
 
       break;
     }
@@ -959,24 +1039,25 @@ void Discovery::DispatchDiscoveryMsg(const std::string &_fromIp, char *_msg)
       advSrv.Unpack(pBody);
 
       // Check scope of the topic.
-      if ((advSrv.GetPublisher().Scope() == Scope_t::Process) ||
-          (advSrv.GetPublisher().Scope() == Scope_t::Host &&
+      if ((advSrv.Publisher().Scope() == Scope_t::PROCESS) ||
+          (advSrv.Publisher().Scope() == Scope_t::HOST &&
            _fromIp != this->dataPtr->hostAddr))
       {
         return;
       }
 
-      SrvDiscoveryCallback cb = this->dataPtr->disconnectionSrvCb;
-
-      if (cb)
+      if (disconnectSrvCb)
       {
         // Notify the new disconnection.
-        cb(advSrv.GetPublisher());
+        disconnectSrvCb(advSrv.Publisher());
       }
 
       // Remove the address entry for this topic.
-      this->dataPtr->infoSrv.DelPublisherByNode(advSrv.GetPublisher().Topic(),
-        advSrv.GetPublisher().PUuid(), advSrv.GetPublisher().NUuid());
+      {
+        std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+        this->dataPtr->infoSrv.DelPublisherByNode(advSrv.Publisher().Topic(),
+          advSrv.Publisher().PUuid(), advSrv.Publisher().NUuid());
+      }
 
       break;
     }
@@ -1061,15 +1142,17 @@ void Discovery::PrintCurrentState() const
 //////////////////////////////////////////////////
 void Discovery::TopicList(std::vector<std::string> &_topics) const
 {
+  this->WaitForInit();
   std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
-  this->dataPtr->infoMsg.GetTopicList(_topics);
+  this->dataPtr->infoMsg.TopicList(_topics);
 }
 
 //////////////////////////////////////////////////
 void Discovery::ServiceList(std::vector<std::string> &_services) const
 {
+  this->WaitForInit();
   std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
-  this->dataPtr->infoSrv.GetTopicList(_services);
+  this->dataPtr->infoSrv.TopicList(_services);
 }
 
 //////////////////////////////////////////////////
@@ -1077,3 +1160,63 @@ std::recursive_mutex& Discovery::Mutex() const
 {
   return this->dataPtr->mutex;
 }
+
+//////////////////////////////////////////////////
+bool Discovery::RegisterNetIface(const std::string &_ip)
+{
+  // Make a new socket for sending discovery information.
+  int sock = static_cast<int>(socket(PF_INET, SOCK_DGRAM, IPPROTO_UDP));
+  if (sock < 0)
+  {
+    std::cerr << "Socket creation failed." << std::endl;
+    return false;
+  }
+
+  // Socket option: IP_MULTICAST_IF.
+  // This socket option needs to be applied to each socket used to send data.
+  // This option selects the source interface for outgoing messages.
+  struct in_addr ifAddr;
+  ifAddr.s_addr = inet_addr(_ip.c_str());
+  if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_IF,
+    reinterpret_cast<const char*>(&ifAddr), sizeof(ifAddr)) != 0)
+  {
+    std::cerr << "Error setting socket option (IP_MULTICAST_IF)." << std::endl;
+    return false;
+  }
+
+  this->dataPtr->sockets.push_back(sock);
+
+  // Join the multicast group. We have to do it for each network interface but
+  // we can do it on the same socket. We will use the socket at position 0 for
+  // receiving multicast information.
+  struct ip_mreq group;
+  group.imr_multiaddr.s_addr =
+    inet_addr(this->dataPtr->MulticastGroup.c_str());
+  group.imr_interface.s_addr = inet_addr(_ip.c_str());
+  if (setsockopt(this->dataPtr->sockets.at(0), IPPROTO_IP, IP_ADD_MEMBERSHIP,
+    reinterpret_cast<const char*>(&group), sizeof(group)) != 0)
+  {
+    std::cerr << "Error setting socket option (IP_ADD_MEMBERSHIP)."
+              << std::endl;
+    return false;
+  }
+
+  return true;
+}
+
+/////////////////////////////////////////////////
+void Discovery::WaitForInit() const
+{
+  bool ready;
+  {
+    std::lock_guard<std::recursive_mutex> lock(this->dataPtr->mutex);
+    ready = this->dataPtr->initialized;
+  }
+
+  if (!ready)
+  {
+    std::unique_lock<std::recursive_mutex> lk(this->dataPtr->mutex);
+    this->dataPtr->initializedCv.wait(
+       lk, [this]{return this->dataPtr->initialized;});
+  }
+}
