@@ -16,10 +16,13 @@
 */
 
 #include <chrono>
+#include <cstdlib>
 #include <memory>
 #include <string>
 #include <thread>
+
 #include "gtest/gtest.h"
+#include "ignition/transport/AdvertiseOptions.hh"
 #include "ignition/transport/Discovery.hh"
 #include "ignition/transport/DiscoveryPrivate.hh"
 #include "ignition/transport/Packet.hh"
@@ -47,7 +50,7 @@ std::string ctrl2   = "tcp://127.0.0.1:12348";
 std::string id2     = "identity2";
 std::string pUuid2  = transport::Uuid().ToString();
 std::string nUuid2  = transport::Uuid().ToString();
-transport::Scope_t scope = transport::Scope_t::All;
+transport::Scope_t scope = transport::Scope_t::ALL;
 bool connectionExecuted = false;
 bool connectionExecutedMF = false;
 bool disconnectionExecuted = false;
@@ -183,7 +186,7 @@ void onDisconnectionSrv(const transport::ServicePublisher &_publisher)
 class MyClass
 {
   /// \brief Class constructor.
-  public: MyClass(const std::string &_pUuid)
+  public: explicit MyClass(const std::string &_pUuid)
   {
     this->discov.reset(new transport::Discovery(_pUuid));
   }
@@ -292,10 +295,10 @@ TEST(DiscoveryTest, TestBasicAPI)
   // Create a discovery node.
   transport::Discovery discovery(pUuid1);
 
-  discovery.SilenceInterval(newSilenceInterval);
-  discovery.ActivityInterval(newActivityInterval);
-  discovery.AdvertiseInterval(newAdvertiseInterval);
-  discovery.HeartbeatInterval(newHeartbeatInterval);
+  discovery.SetSilenceInterval(newSilenceInterval);
+  discovery.SetActivityInterval(newActivityInterval);
+  discovery.SetAdvertiseInterval(newAdvertiseInterval);
+  discovery.SetHeartbeatInterval(newHeartbeatInterval);
   EXPECT_EQ(discovery.SilenceInterval(), newSilenceInterval);
   EXPECT_EQ(discovery.ActivityInterval(), newActivityInterval);
   EXPECT_EQ(discovery.AdvertiseInterval(), newAdvertiseInterval);
@@ -405,7 +408,7 @@ TEST(DiscoveryTest, TestAdvertise)
   // This should not trigger a discovery response on discovery2. They are in
   // different proccesses and the scope is set to "Process".
   transport::MessagePublisher publisher2("/topic2", addr1, ctrl1, pUuid1,
-    nUuid1, transport::Scope_t::Process, "type");
+    nUuid1, transport::Scope_t::PROCESS, "type");
   EXPECT_TRUE(discovery1.AdvertiseMsg(publisher2));
 
   waitForCallback(MaxIters, Nap, connectionExecuted);
@@ -417,7 +420,7 @@ TEST(DiscoveryTest, TestAdvertise)
 
   // This should trigger a discovery response on discovery2.
   transport::MessagePublisher publisher3("/topic3", addr1, ctrl1, pUuid1,
-    nUuid1, transport::Scope_t::Host, "type");
+    nUuid1, transport::Scope_t::HOST, "type");
   EXPECT_TRUE(discovery1.AdvertiseMsg(publisher3));
 
   waitForCallback(MaxIters, Nap, connectionExecuted);
@@ -941,3 +944,30 @@ TEST(DiscoveryTest, TestActivity)
   // We shouldn't observe activity from proc1Uuid2 anymore.
   discovery1.TestActivity(proc2Uuid, false);
 }
+
+//////////////////////////////////////////////////
+/// \brief Check that a wrong IGN_IP value makes HostAddr() to return 127.0.0.1
+TEST(DiscoveryTest, WrongIgnIp)
+{
+  // Save the current value of IGN_IP environment variable.
+  char *ipEnv;
+#ifdef _MSC_VER
+  size_t sz = 0;
+  _dupenv_s(&ipEnv, &sz, "IGN_IP");
+#else
+  ipEnv = std::getenv("IGN_IP");
+#endif
+
+  // Incorrect value for IGN_IP
+  setenv("IGN_IP", "127.0.0.0", 1);
+
+  transport::Discovery discovery1(pUuid1);
+  EXPECT_EQ(discovery1.HostAddr(), "127.0.0.1");
+
+  // Unset IGN_IP.
+  unsetenv("IGN_IP");
+
+  // Restore IGN_IP.
+  if (ipEnv)
+    setenv("IGN_IP", ipEnv, 1);
+}
