@@ -17,11 +17,13 @@
 
 #include <map>
 #include <string>
+
 #include "ignition/transport/HandlerStorage.hh"
 #include "ignition/transport/RepHandler.hh"
 #include "ignition/transport/SubscriptionHandler.hh"
 #include "ignition/transport/TransportTypes.hh"
 #include "gtest/gtest.h"
+
 #include "msgs/int.pb.h"
 #include "msgs/vector3d.pb.h"
 
@@ -44,13 +46,12 @@ void reset()
 
 //////////////////////////////////////////////////
 /// \brief Callback providing a service call.
-void cb1(const std::string &_topic, const transport::msgs::Vector3d &_req,
-  transport::msgs::Int &_rep, bool &_result)
+void cb1(const transport::msgs::Vector3d &_req, transport::msgs::Int &_rep,
+  bool &_result)
 {
-  EXPECT_EQ(_topic, topic);
-  EXPECT_FLOAT_EQ(_req.x(), 1.0);
-  EXPECT_FLOAT_EQ(_req.y(), 2.0);
-  EXPECT_FLOAT_EQ(_req.z(), 3.0);
+  EXPECT_DOUBLE_EQ(_req.x(), 1.0);
+  EXPECT_DOUBLE_EQ(_req.y(), 2.0);
+  EXPECT_DOUBLE_EQ(_req.z(), 3.0);
   _rep.set_data(intResult);
   _result = true;
 
@@ -66,16 +67,18 @@ TEST(RepStorageTest, RepStorageAPI)
   transport::HandlerStorage<transport::IRepHandler> reps;
   transport::msgs::Int rep1Msg;
   bool result;
-
   transport::msgs::Vector3d reqMsg;
+  std::string reqType = reqMsg.GetTypeName();
+  std::string rep1Type = rep1Msg.GetTypeName();
+
   reqMsg.set_x(1.0);
   reqMsg.set_y(2.0);
   reqMsg.set_z(3.0);
 
   // Check some operations when there is no data stored.
-  EXPECT_FALSE(reps.GetHandlers(topic, m));
-  EXPECT_FALSE(reps.GetHandler(topic, handler));
-  EXPECT_FALSE(reps.GetHandler(topic, nUuid1, hUuid, handler));
+  EXPECT_FALSE(reps.Handlers(topic, m));
+  EXPECT_FALSE(reps.FirstHandler(topic, reqType, rep1Type, handler));
+  EXPECT_FALSE(reps.Handler(topic, nUuid1, hUuid, handler));
   EXPECT_FALSE(reps.HasHandlersForTopic(topic));
   EXPECT_FALSE(reps.RemoveHandlersForNode(topic, nUuid1));
   EXPECT_FALSE(reps.HasHandlersForNode(topic, nUuid1));
@@ -85,20 +88,21 @@ TEST(RepStorageTest, RepStorageAPI)
     transport::msgs::Int>> rep1HandlerPtr(new transport::RepHandler<
       transport::msgs::Vector3d, transport::msgs::Int>());
 
-  rep1HandlerPtr->Callback(cb1);
+  rep1HandlerPtr->SetCallback(cb1);
 
   // Insert the handler and check operations.
   reps.AddHandler(topic, nUuid1, rep1HandlerPtr);
   EXPECT_TRUE(reps.HasHandlersForTopic(topic));
   EXPECT_TRUE(reps.HasHandlersForNode(topic, nUuid1));
   EXPECT_FALSE(reps.HasHandlersForNode(topic, nUuid2));
-  EXPECT_TRUE(reps.GetHandler(topic, handler));
+  EXPECT_TRUE(reps.FirstHandler(topic, reqType, rep1Type, handler));
+  ASSERT_TRUE(handler != NULL);
   std::string handlerUuid = handler->HandlerUuid();
   EXPECT_EQ(handlerUuid, rep1HandlerPtr->HandlerUuid());
-  EXPECT_TRUE(reps.GetHandler(topic, nUuid1, handlerUuid, handler));
-  EXPECT_FALSE(reps.GetHandler(topic, "wrongNodeUuid", handlerUuid, handler));
-  EXPECT_FALSE(reps.GetHandler(topic, nUuid1, "wrongHandlerUuid", handler));
-  EXPECT_TRUE(reps.GetHandlers(topic, m));
+  EXPECT_TRUE(reps.Handler(topic, nUuid1, handlerUuid, handler));
+  EXPECT_FALSE(reps.Handler(topic, "wrongNodeUuid", handlerUuid, handler));
+  EXPECT_FALSE(reps.Handler(topic, nUuid1, "wrongHandlerUuid", handler));
+  EXPECT_TRUE(reps.Handlers(topic, m));
   EXPECT_EQ(m.size(), 1u);
   EXPECT_EQ(m.begin()->first, nUuid1);
 
@@ -106,7 +110,7 @@ TEST(RepStorageTest, RepStorageAPI)
 
   // Check the handler operations.
   handler = m[nUuid1].begin()->second;
-  handler->RunLocalCallback(topic, reqMsg, rep1Msg, result);
+  handler->RunLocalCallback(reqMsg, rep1Msg, result);
   EXPECT_TRUE(cbExecuted);
   EXPECT_EQ(rep1Msg.data(), intResult);
   EXPECT_TRUE(result);
@@ -116,7 +120,7 @@ TEST(RepStorageTest, RepStorageAPI)
   std::string reqSerialized;
   std::string repSerialized;
   EXPECT_TRUE(reqMsg.SerializeToString(&reqSerialized));
-  handler->RunCallback(topic, reqSerialized, repSerialized, result);
+  handler->RunCallback(reqSerialized, repSerialized, result);
   EXPECT_TRUE(cbExecuted);
   EXPECT_TRUE(result);
   EXPECT_TRUE(rep1Msg.ParseFromString(repSerialized));
@@ -149,24 +153,24 @@ TEST(RepStorageTest, RepStorageAPI)
   EXPECT_TRUE(reps.HasHandlersForTopic(topic));
   EXPECT_TRUE(reps.HasHandlersForNode(topic, nUuid1));
   EXPECT_TRUE(reps.HasHandlersForNode(topic, nUuid2));
-  EXPECT_TRUE(reps.GetHandler(topic, handler));
+  EXPECT_TRUE(reps.FirstHandler(topic, reqType, rep1Type, handler));
   handlerUuid = rep3HandlerPtr->HandlerUuid();
-  EXPECT_TRUE(reps.GetHandler(topic, nUuid2, handlerUuid, handler));
+  EXPECT_TRUE(reps.Handler(topic, nUuid2, handlerUuid, handler));
   EXPECT_EQ(handler->HandlerUuid(), handlerUuid);
-  EXPECT_TRUE(reps.GetHandlers(topic, m));
+  EXPECT_TRUE(reps.Handlers(topic, m));
   EXPECT_EQ(m.size(), 2u);
 
   reset();
 
   // Check the handler operations.
   handler = m[nUuid2].begin()->second;
-  handler->RunLocalCallback(topic, reqMsg, rep1Msg, result);
+  handler->RunLocalCallback(reqMsg, rep1Msg, result);
   EXPECT_FALSE(cbExecuted);
   EXPECT_FALSE(result);
 
   reset();
 
-  handler->RunCallback(topic, reqSerialized, repSerialized, result);
+  handler->RunCallback(reqSerialized, repSerialized, result);
   EXPECT_FALSE(cbExecuted);
   EXPECT_FALSE(result);
 
@@ -175,7 +179,7 @@ TEST(RepStorageTest, RepStorageAPI)
   EXPECT_TRUE(reps.HasHandlersForTopic(topic));
   EXPECT_TRUE(reps.HasHandlersForNode(topic, nUuid1));
   EXPECT_FALSE(reps.HasHandlersForNode(topic, nUuid2));
-  EXPECT_TRUE(reps.GetHandlers(topic, m));
+  EXPECT_TRUE(reps.Handlers(topic, m));
   EXPECT_EQ(m.size(), 1u);
   EXPECT_EQ(m.begin()->first, nUuid1);
 
@@ -183,7 +187,7 @@ TEST(RepStorageTest, RepStorageAPI)
 
   // Remove all REP handlers for node1.
   EXPECT_TRUE(reps.RemoveHandlersForNode(topic, nUuid1));
-  EXPECT_FALSE(reps.GetHandlers(topic, m));
+  EXPECT_FALSE(reps.Handlers(topic, m));
   EXPECT_FALSE(reps.HasHandlersForTopic(topic));
   EXPECT_FALSE(reps.RemoveHandlersForNode(topic, nUuid1));
   EXPECT_FALSE(reps.HasHandlersForNode(topic, nUuid1));
@@ -207,7 +211,6 @@ TEST(RepStorageTest, RepStorageAPI)
 /// registering a callback, and then, we try to execute the callback.
 TEST(RepStorageTest, SubStorageNoCallbacks)
 {
-  transport::ISubscriptionHandlerPtr handler;
   transport::HandlerStorage<transport::ISubscriptionHandler> subs;
   transport::msgs::Int msg;
   msg.set_data(5);
@@ -222,8 +225,20 @@ TEST(RepStorageTest, SubStorageNoCallbacks)
 
   transport::ISubscriptionHandlerPtr h;
   std::string handlerUuid = sub1HandlerPtr->HandlerUuid();
-  EXPECT_TRUE(subs.GetHandler(topic, nUuid1, handlerUuid, h));
-  EXPECT_FALSE(h->RunLocalCallback(topic, msg));
+  EXPECT_TRUE(subs.Handler(topic, nUuid1, handlerUuid, h));
+  EXPECT_FALSE(h->RunLocalCallback(msg));
+
+  // Try to retrieve the first callback with an incorrect type.
+  transport::ISubscriptionHandlerPtr handler;
+  EXPECT_FALSE(subs.FirstHandler(topic, "incorrect type", handler));
+
+  // Now try to retrieve the first callback with the correct type.
+  EXPECT_TRUE(subs.FirstHandler(topic, msg.GetTypeName(), handler));
+
+  // Verify the handler.
+  EXPECT_EQ(handler->TypeName(), sub1HandlerPtr->TypeName());
+  EXPECT_EQ(handler->NodeUuid(), sub1HandlerPtr->NodeUuid());
+  EXPECT_EQ(handler->HandlerUuid(), sub1HandlerPtr->HandlerUuid());
 }
 
 //////////////////////////////////////////////////
