@@ -16,77 +16,90 @@
 */
 
 #ifdef _MSC_VER
-# pragma warning(push, 0)
+#pragma warning(push, 0)
 #endif
 #include <google/protobuf/message.h>
-#include <cassert>
-#include <cstdlib>
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
+
 #include <algorithm>
+#include <cassert>
+#include <csignal>
+#include <condition_variable>
 #include <iostream>
 #include <map>
 #include <mutex>
 #include <string>
 #include <unordered_set>
 #include <vector>
-#ifdef _MSC_VER
-# pragma warning(pop)
-#endif
+
 #include "ignition/transport/Node.hh"
+#include "ignition/transport/NodeOptions.hh"
 #include "ignition/transport/NodePrivate.hh"
 #include "ignition/transport/NodeShared.hh"
 #include "ignition/transport/TopicUtils.hh"
 #include "ignition/transport/TransportTypes.hh"
 #include "ignition/transport/Uuid.hh"
 
+#ifdef _MSC_VER
+#pragma warning(disable: 4503)
+#endif
+
 using namespace ignition;
 using namespace transport;
 
-//////////////////////////////////////////////////
-Node::Node()
-  : dataPtr(new NodePrivate())
+namespace ignition
 {
-  // Check if the environment variable IGN_PARTITION is present.
-  std::string partitionStr;
-  char *envPartition = std::getenv("IGN_PARTITION");
-
-  if (envPartition)
+  namespace transport
   {
-    partitionStr = std::string(envPartition);
-    if (TopicUtils::IsValidNamespace(partitionStr))
-      this->dataPtr->partition = partitionStr;
-    else
-      std::cerr << "Invalid IGN_PARTITION value [" << partitionStr << "]"
-                << std::endl;
-  }
+    /// \brief Flag to detect SIGINT or SIGTERM while the code is executing
+    /// waitForShutdown().
+    static bool g_shutdown = false;
 
-  // Generate the node UUID.
-  Uuid uuid;
-  this->dataPtr->nUuid = uuid.ToString();
+    /// \brief Mutex to protect the boolean shutdown variable.
+    static std::mutex g_shutdown_mutex;
+
+    /// \brief Condition variable to wakeup waitForShutdown() and exit.
+    static std::condition_variable g_shutdown_cv;
+
+    //////////////////////////////////////////////////
+    /// \brief Function executed when a SIGINT or SIGTERM signals are captured.
+    /// \param[in] _signal Signal received.
+    static void signal_handler(const int _signal)
+    {
+      if (_signal == SIGINT || _signal == SIGTERM)
+      {
+        g_shutdown_mutex.lock();
+        g_shutdown = true;
+        g_shutdown_mutex.unlock();
+        g_shutdown_cv.notify_all();
+      }
+    }
+  }
 }
 
 //////////////////////////////////////////////////
-Node::Node(const std::string &_partition, const std::string &_ns)
-  : Node()
+void ignition::transport::waitForShutdown()
 {
-  if (TopicUtils::IsValidNamespace(_ns))
-    this->dataPtr->ns = _ns;
-  else
-  {
-    std::cerr << "Namespace [" << _ns << "] is not valid." << std::endl;
-    std::cerr << "Using default namespace." << std::endl;
-  }
+  // Install a signal handler for SIGINT and SIGTERM.
+  std::signal(SIGINT,  signal_handler);
+  std::signal(SIGTERM, signal_handler);
 
-  if (TopicUtils::IsValidNamespace(_partition))
-    this->dataPtr->partition = _partition;
-  else
-  {
-    std::cerr << "Partition [" << _partition << "] is not valid." << std::endl;
-    std::cerr << "Using default partition." << std::endl;
-  }
+  std::unique_lock<std::mutex> lk(g_shutdown_mutex);
+  g_shutdown_cv.wait(lk, []{return g_shutdown;});
+}
 
+//////////////////////////////////////////////////
+Node::Node(const NodeOptions &_options)
+  : dataPtr(new NodePrivate())
+{
   // Generate the node UUID.
   Uuid uuid;
   this->dataPtr->nUuid = uuid.ToString();
+
+  // Save the options.
+  this->dataPtr->options = _options;
 }
 
 //////////////////////////////////////////////////
@@ -129,41 +142,6 @@ Node::~Node()
   assert(this->AdvertisedServices().empty());
 }
 
-//////////////////////////////////////////////////
-bool Node::Advertise(const std::string &_topic, const Scope_t &_scope)
-{
-  std::string fullyQualifiedTopic;
-  if (!TopicUtils::GetFullyQualifiedName(this->dataPtr->partition,
-    this->dataPtr->ns, _topic, fullyQualifiedTopic))
-  {
-    std::cerr << "Topic [" << _topic << "] is not valid." << std::endl;
-    return false;
-  }
-
-  std::lock(this->Shared()->discovery->Mutex(), this->dataPtr->shared->mutex);
-  std::lock_guard<std::recursive_mutex> discLk(
-    this->Shared()->discovery->Mutex(), std::adopt_lock);
-  std::lock_guard<std::recursive_mutex> lk(
-    this->dataPtr->shared->mutex, std::adopt_lock);
-
-  // Add the topic to the list of advertised topics (if it was not before).
-  this->dataPtr->topicsAdvertised.insert(fullyQualifiedTopic);
-
-  // Notify the discovery service to register and advertise my topic.
-  MessagePublisher publisher(fullyQualifiedTopic,
-    this->dataPtr->shared->myAddress, this->dataPtr->shared->myControlAddress,
-    this->dataPtr->shared->pUuid, this->dataPtr->nUuid, _scope, "unused");
-
-  if (!this->dataPtr->shared->discovery->AdvertiseMsg(publisher))
-  {
-    std::cerr << "Node::Advertise(): Error advertising a topic. "
-              << "Did you forget to start the discovery service?" << std::endl;
-    return false;
-  }
-
-  return true;
-}
-
 //////////////////////////////////////////////////
 std::vector<std::string> Node::AdvertisedTopics() const
 {
@@ -185,18 +163,14 @@ std::vector<std::string> Node::AdvertisedTopics() const
 bool Node::Unadvertise(const std::string &_topic)
 {
   std::string fullyQualifiedTopic = _topic;
-  if (!TopicUtils::GetFullyQualifiedName(this->dataPtr->partition,
-    this->dataPtr->ns, _topic, fullyQualifiedTopic))
+  if (!TopicUtils::FullyQualifiedName(this->Options().Partition(),
+    this->Options().NameSpace(), _topic, fullyQualifiedTopic))
   {
     std::cerr << "Topic [" << _topic << "] is not valid." << std::endl;
     return false;
   }
 
-  std::lock(this->Shared()->discovery->Mutex(), this->dataPtr->shared->mutex);
-  std::lock_guard<std::recursive_mutex> discLk(
-    this->Shared()->discovery->Mutex(), std::adopt_lock);
-  std::lock_guard<std::recursive_mutex> lk(
-    this->dataPtr->shared->mutex, std::adopt_lock);
+  std::lock_guard<std::recursive_mutex> lk(this->dataPtr->shared->mutex);
 
   // Remove the topic from the list of advertised topics in this node.
   this->dataPtr->topicsAdvertised.erase(fullyQualifiedTopic);
@@ -215,26 +189,55 @@ bool Node::Unadvertise(const std::string &_topic)
 bool Node::Publish(const std::string &_topic, const ProtoMsg &_msg)
 {
   std::string fullyQualifiedTopic;
-  if (!TopicUtils::GetFullyQualifiedName(this->dataPtr->partition,
-    this->dataPtr->ns, _topic, fullyQualifiedTopic))
+  if (!TopicUtils::FullyQualifiedName(this->Options().Partition(),
+    this->Options().NameSpace(), _topic, fullyQualifiedTopic))
   {
     std::cerr << "Topic [" << _topic << "] is not valid." << std::endl;
     return false;
   }
 
-  std::lock_guard<std::recursive_mutex> lk(this->dataPtr->shared->mutex);
-
-  // Topic not advertised before.
-  if (this->dataPtr->topicsAdvertised.find(fullyQualifiedTopic) ==
-      this->dataPtr->topicsAdvertised.end())
+  std::map<std::string, ISubscriptionHandler_M> handlers;
+  bool hasLocalSubscribers;
+  bool hasRemoteSubscribers;
   {
+    std::lock_guard<std::recursive_mutex> lk(this->dataPtr->shared->mutex);
+
+    // Topic not advertised before.
+    if (this->dataPtr->topicsAdvertised.find(fullyQualifiedTopic) ==
+        this->dataPtr->topicsAdvertised.end())
+    {
+      return false;
+    }
+    hasLocalSubscribers =
+      this->dataPtr->shared->localSubscriptions.Handlers(fullyQualifiedTopic,
+        handlers);
+    hasRemoteSubscribers =
+      this->dataPtr->shared->remoteSubscribers.HasTopic(fullyQualifiedTopic);
+  }
+
+  // Check that the msg type matches the type previously advertised
+  // for topic '_topic'.
+  MessagePublisher pub;
+  auto &info = this->dataPtr->shared->discovery->DiscoveryMsgInfo();
+  std::string procUuid = this->dataPtr->shared->pUuid;
+  std::string nodeUuid = this->dataPtr->nUuid;
+  if (!info.Publisher(fullyQualifiedTopic, procUuid, nodeUuid, pub))
+  {
+    std::cerr << "Node::Publish() I cannot find the msgType registered for "
+              << "topic [" << _topic << "]" << std::endl;
+    return false;
+  }
+
+  if (pub.MsgTypeName() != _msg.GetTypeName())
+  {
+    std::cerr << "Node::Publish() Type mismatch." << std::endl
+              << "\t* Type advertised: " << pub.MsgTypeName() << std::endl
+              << "\t* Type published: " << _msg.GetTypeName() << std::endl;
     return false;
   }
 
   // Local subscribers.
-  std::map<std::string, ISubscriptionHandler_M> handlers;
-  if (this->dataPtr->shared->localSubscriptions.GetHandlers(fullyQualifiedTopic,
-        handlers))
+  if (hasLocalSubscribers)
   {
     for (auto &node : handlers)
     {
@@ -243,7 +246,12 @@ bool Node::Publish(const std::string &_topic, const ProtoMsg &_msg)
         ISubscriptionHandlerPtr subscriptionHandlerPtr = handler.second;
 
         if (subscriptionHandlerPtr)
-          subscriptionHandlerPtr->RunLocalCallback(fullyQualifiedTopic, _msg);
+        {
+          if (subscriptionHandlerPtr->TypeName() != _msg.GetTypeName())
+            continue;
+
+          subscriptionHandlerPtr->RunLocalCallback(_msg);
+        }
         else
         {
           std::cerr << "Node::Publish(): Subscription handler is NULL"
@@ -254,7 +262,7 @@ bool Node::Publish(const std::string &_topic, const ProtoMsg &_msg)
   }
 
   // Remote subscribers.
-  if (this->dataPtr->shared->remoteSubscribers.HasTopic(fullyQualifiedTopic))
+  if (hasRemoteSubscribers)
   {
     std::string data;
     if (!_msg.SerializeToString(&data))
@@ -263,7 +271,8 @@ bool Node::Publish(const std::string &_topic, const ProtoMsg &_msg)
       return false;
     }
 
-    this->dataPtr->shared->Publish(fullyQualifiedTopic, data);
+    this->dataPtr->shared->Publish(fullyQualifiedTopic, data,
+      _msg.GetTypeName());
   }
   // Debug output.
   // else
@@ -294,18 +303,14 @@ std::vector<std::string> Node::SubscribedTopics() const
 bool Node::Unsubscribe(const std::string &_topic)
 {
   std::string fullyQualifiedTopic;
-  if (!TopicUtils::GetFullyQualifiedName(this->dataPtr->partition,
-    this->dataPtr->ns, _topic, fullyQualifiedTopic))
+  if (!TopicUtils::FullyQualifiedName(this->Options().Partition(),
+    this->Options().NameSpace(), _topic, fullyQualifiedTopic))
   {
     std::cerr << "Topic [" << _topic << "] is not valid." << std::endl;
     return false;
   }
 
-  std::lock(this->Shared()->discovery->Mutex(), this->dataPtr->shared->mutex);
-  std::lock_guard<std::recursive_mutex> discLk(
-    this->Shared()->discovery->Mutex(), std::adopt_lock);
-  std::lock_guard<std::recursive_mutex> lk(
-    this->dataPtr->shared->mutex, std::adopt_lock);
+  std::lock_guard<std::recursive_mutex> lk(this->dataPtr->shared->mutex);
 
   this->dataPtr->shared->localSubscriptions.RemoveHandlersForNode(
     fullyQualifiedTopic, this->dataPtr->nUuid);
@@ -390,18 +395,14 @@ std::vector<std::string> Node::AdvertisedServices() const
 bool Node::UnadvertiseSrv(const std::string &_topic)
 {
   std::string fullyQualifiedTopic;
-  if (!TopicUtils::GetFullyQualifiedName(this->dataPtr->partition,
-    this->dataPtr->ns, _topic, fullyQualifiedTopic))
+  if (!TopicUtils::FullyQualifiedName(this->Options().Partition(),
+    this->Options().NameSpace(), _topic, fullyQualifiedTopic))
   {
     std::cerr << "Service [" << _topic << "] is not valid." << std::endl;
     return false;
   }
 
-  std::lock(this->Shared()->discovery->Mutex(), this->dataPtr->shared->mutex);
-  std::lock_guard<std::recursive_mutex> discLk(
-    this->Shared()->discovery->Mutex(), std::adopt_lock);
-  std::lock_guard<std::recursive_mutex> lk(
-    this->dataPtr->shared->mutex, std::adopt_lock);
+  std::lock_guard<std::recursive_mutex> lk(this->dataPtr->shared->mutex);
 
   // Remove the topic from the list of advertised topics in this node.
   this->dataPtr->srvsAdvertised.erase(fullyQualifiedTopic);
@@ -426,12 +427,6 @@ void Node::TopicList(std::vector<std::string> &_topics) const
   std::vector<std::string> allTopics;
   _topics.clear();
 
-  std::lock(this->Shared()->discovery->Mutex(), this->dataPtr->shared->mutex);
-  std::lock_guard<std::recursive_mutex> discLk(
-    this->Shared()->discovery->Mutex(), std::adopt_lock);
-  std::lock_guard<std::recursive_mutex> lk(
-    this->dataPtr->shared->mutex, std::adopt_lock);
-
   this->dataPtr->shared->discovery->TopicList(allTopics);
 
   for (auto &topic : allTopics)
@@ -443,7 +438,7 @@ void Node::TopicList(std::vector<std::string> &_topics) const
       partition.erase(partition.begin());
 
     // Discard if the partition name does not match this node's partition.
-    if (partition != this->Partition())
+    if (partition != this->Options().Partition())
       continue;
 
     // Remove the partition part from the topic.
@@ -459,12 +454,6 @@ void Node::ServiceList(std::vector<std::string> &_services) const
   std::vector<std::string> allServices;
   _services.clear();
 
-  std::lock(this->Shared()->discovery->Mutex(), this->dataPtr->shared->mutex);
-  std::lock_guard<std::recursive_mutex> discLk(
-    this->Shared()->discovery->Mutex(), std::adopt_lock);
-  std::lock_guard<std::recursive_mutex> lk(
-    this->dataPtr->shared->mutex, std::adopt_lock);
-
   this->dataPtr->shared->discovery->ServiceList(allServices);
 
   for (auto &service : allServices)
@@ -476,7 +465,7 @@ void Node::ServiceList(std::vector<std::string> &_services) const
       partition.erase(partition.begin());
 
     // Discard if the partition name does not match this node's partition.
-    if (partition != this->Partition())
+    if (partition != this->Options().Partition())
       continue;
 
     // Remove the partition part from the service.
@@ -487,37 +476,125 @@ void Node::ServiceList(std::vector<std::string> &_services) const
 }
 
 //////////////////////////////////////////////////
-const std::string& Node::Partition() const
-{
-  return this->dataPtr->partition;
-}
-
-//////////////////////////////////////////////////
-const std::string& Node::NameSpace() const
-{
-  return this->dataPtr->ns;
-}
-
-//////////////////////////////////////////////////
-NodeShared* Node::Shared() const
+NodeShared *Node::Shared() const
 {
   return this->dataPtr->shared;
 }
 
 //////////////////////////////////////////////////
-const std::string& Node::NodeUuid() const
+const std::string &Node::NodeUuid() const
 {
   return this->dataPtr->nUuid;
 }
 
 //////////////////////////////////////////////////
-std::unordered_set<std::string>& Node::TopicsSubscribed() const
+std::unordered_set<std::string> &Node::TopicsAdvertised() const
+{
+  return this->dataPtr->topicsAdvertised;
+}
+
+//////////////////////////////////////////////////
+std::unordered_set<std::string> &Node::TopicsSubscribed() const
 {
   return this->dataPtr->topicsSubscribed;
 }
 
 //////////////////////////////////////////////////
-std::unordered_set<std::string>& Node::SrvsAdvertised() const
+std::unordered_set<std::string> &Node::SrvsAdvertised() const
 {
   return this->dataPtr->srvsAdvertised;
 }
+
+//////////////////////////////////////////////////
+NodeOptions &Node::Options() const
+{
+  return this->dataPtr->options;
+}
+
+//////////////////////////////////////////////////
+bool Node::TopicInfo(const std::string &_topic,
+                     std::vector<MessagePublisher> &_publishers) const
+{
+  this->dataPtr->shared->discovery->WaitForInit();
+
+  // Construct a topic name with the partition and namespace
+  std::string fullyQualifiedTopic;
+  if (!TopicUtils::FullyQualifiedName(this->Options().Partition(),
+    this->Options().NameSpace(), _topic, fullyQualifiedTopic))
+  {
+    return false;
+  }
+
+  std::lock_guard<std::recursive_mutex> lk(this->dataPtr->shared->mutex);
+
+  // Get all the publishers on the given topics
+  MsgAddresses_M pubs;
+  if (!this->dataPtr->shared->discovery->MsgPublishers(
+        fullyQualifiedTopic, pubs))
+  {
+    return false;
+  }
+
+  _publishers.clear();
+
+  // Copy the publishers.
+  for (MsgAddresses_M::iterator iter = pubs.begin(); iter != pubs.end(); ++iter)
+  {
+    for (std::vector<MessagePublisher>::iterator pubIter = iter->second.begin();
+         pubIter != iter->second.end(); ++pubIter)
+    {
+      // Add the publisher if it doesn't already exist.
+      if (std::find(_publishers.begin(), _publishers.end(), *pubIter) ==
+          _publishers.end())
+      {
+        _publishers.push_back(*pubIter);
+      }
+    }
+  }
+
+  return true;
+}
+
+//////////////////////////////////////////////////
+bool Node::ServiceInfo(const std::string &_service,
+                       std::vector<ServicePublisher> &_publishers) const
+{
+  this->dataPtr->shared->discovery->WaitForInit();
+
+  // Construct a topic name with the partition and namespace
+  std::string fullyQualifiedTopic;
+  if (!TopicUtils::FullyQualifiedName(this->Options().Partition(),
+    this->Options().NameSpace(), _service, fullyQualifiedTopic))
+  {
+    return false;
+  }
+
+  std::lock_guard<std::recursive_mutex> lk(this->dataPtr->shared->mutex);
+
+  // Get all the publishers on the given service.
+  SrvAddresses_M pubs;
+  if (!this->dataPtr->shared->discovery->SrvPublishers(
+        fullyQualifiedTopic, pubs))
+  {
+    return false;
+  }
+
+  _publishers.clear();
+
+  // Copy the publishers.
+  for (SrvAddresses_M::iterator iter = pubs.begin(); iter != pubs.end(); ++iter)
+  {
+    for (std::vector<ServicePublisher>::iterator pubIter = iter->second.begin();
+         pubIter != iter->second.end(); ++pubIter)
+    {
+      // Add the publisher if it doesn't already exist.
+      if (std::find(_publishers.begin(), _publishers.end(), *pubIter) ==
+          _publishers.end())
+      {
+        _publishers.push_back(*pubIter);
+      }
+    }
+  }
+
+  return true;
+}
