@@ -16,9 +16,13 @@
 */
 
 #ifdef _MSC_VER
-# pragma warning(push, 0)
+#pragma warning(push, 0)
 #endif
 #include <zmq.hpp>
+#ifdef _MSC_VER
+#pragma warning(pop)
+#endif
+
 #include <chrono>
 #include <cstdlib>
 #include <iostream>
@@ -27,10 +31,9 @@
 #include <string>
 #include <thread>
 #include <vector>
-#ifdef _MSC_VER
-# pragma warning(pop)
-#endif
+
 #include "ignition/transport/Discovery.hh"
+#include "ignition/transport/Helpers.hh"
 #include "ignition/transport/NodeShared.hh"
 #include "ignition/transport/Packet.hh"
 #include "ignition/transport/RepHandler.hh"
@@ -39,11 +42,15 @@
 #include "ignition/transport/TransportTypes.hh"
 #include "ignition/transport/Uuid.hh"
 
+#ifdef _MSC_VER
+# pragma warning(disable: 4503)
+#endif
+
 using namespace ignition;
 using namespace transport;
 
 //////////////////////////////////////////////////
-NodeShared *NodeShared::GetInstance()
+NodeShared *NodeShared::Instance()
 {
   static NodeShared instance;
   return &instance;
@@ -63,9 +70,8 @@ NodeShared::NodeShared()
     exit(false)
 {
   // If IGN_VERBOSE=1 enable the verbose mode.
-  char const *tmp = std::getenv("IGN_VERBOSE");
-  if (tmp)
-    this->verbose = std::string(tmp) == "1";
+  std::string ignVerbose;
+  this->verbose = (env("IGN_VERBOSE", ignVerbose) && ignVerbose == "1");
 
   char bindEndPoint[1024];
 
@@ -178,6 +184,12 @@ NodeShared::~NodeShared()
   if (this->threadReception.joinable())
     this->threadReception.join();
 
+  // Explicitly reset discovery to prevent callbacks.
+  // See https://bitbucket.org/osrf/gazebo/issues/1976/ and
+  // https://bitbucket.org/ignitionrobotics/ign-transport/pull-requests/139.
+  // Do not merge forward.
+  this->discovery.reset();
+
   // We explicitly destroy the ZMQ socket before destroying the ZMQ context.
   publisher.reset();
   subscriber.reset();
@@ -187,16 +199,23 @@ NodeShared::~NodeShared()
   replier.reset();
   delete this->context;
 #else
-  while (true)
+  bool exitLoop = false;
+  while (!exitLoop)
   {
     std::lock_guard<std::mutex> lock(this->exitMutex);
     {
       if (this->threadReceptionExiting)
-        break;
+        exitLoop = true;
     }
     std::this_thread::sleep_for(std::chrono::milliseconds(50));
   }
 
+  // Explicitly reset discovery to prevent callbacks.
+  // See https://bitbucket.org/osrf/gazebo/issues/1976/ and
+  // https://bitbucket.org/ignitionrobotics/ign-transport/pull-requests/139.
+  // Do not merge forward.
+  this->discovery.reset();
+
   // We intentionally don't destroy the context in Windows.
   // For some reason, when MATLAB deallocates the MEX file makes the context
   // destructor to hang (probably waiting for ZMQ sockets to terminate).
@@ -207,7 +226,8 @@ NodeShared::~NodeShared()
 //////////////////////////////////////////////////
 void NodeShared::RunReceptionTask()
 {
-  while (true)
+  bool exitLoop = false;
+  while (!exitLoop)
   {
     // Poll socket for a reply, with timeout.
     zmq::pollitem_t items[] =
@@ -217,7 +237,14 @@ void NodeShared::RunReceptionTask()
       {static_cast<void*>(*this->replier), 0, ZMQ_POLLIN, 0},
       {static_cast<void*>(*this->responseReceiver), 0, ZMQ_POLLIN, 0}
     };
-    zmq::poll(&items[0], sizeof(items) / sizeof(items[0]), this->timeout);
+    try
+    {
+      zmq::poll(&items[0], sizeof(items) / sizeof(items[0]), this->timeout);
+    }
+    catch(...)
+    {
+      continue;
+    }
 
     //  If we got a reply, process it.
     if (items[0].revents & ZMQ_POLLIN)
@@ -233,7 +260,7 @@ void NodeShared::RunReceptionTask()
     {
       std::lock_guard<std::mutex> lock(this->exitMutex);
       if (this->exit)
-        break;
+        exitLoop = true;
     }
   }
 #ifdef _WIN32
@@ -245,10 +272,12 @@ void NodeShared::RunReceptionTask()
 }
 
 //////////////////////////////////////////////////
-bool NodeShared::Publish(const std::string &_topic, const std::string &_data)
+bool NodeShared::Publish(const std::string &_topic, const std::string &_data,
+  const std::string &_msgType)
 {
   try
   {
+    std::lock_guard<std::recursive_mutex> lock(this->mutex);
     zmq::message_t msg;
     msg.rebuild(_topic.size());
     memcpy(msg.data(), _topic.data(), _topic.size());
@@ -260,6 +289,10 @@ bool NodeShared::Publish(const std::string &_topic, const std::string &_data)
 
     msg.rebuild(_data.size());
     memcpy(msg.data(), _data.data(), _data.size());
+    this->publisher->send(msg, ZMQ_SNDMORE);
+
+    msg.rebuild(_msgType.size());
+    memcpy(msg.data(), _msgType.data(), _msgType.size());
     this->publisher->send(msg, 0);
   }
   catch(const zmq::error_t& ze)
@@ -274,47 +307,52 @@ bool NodeShared::Publish(const std::string &_topic, const std::string &_data)
 //////////////////////////////////////////////////
 void NodeShared::RecvMsgUpdate()
 {
-  std::lock_guard<std::recursive_mutex> lock(this->mutex);
-
   zmq::message_t msg(0);
   std::string topic;
   // std::string sender;
   std::string data;
-
-  try
-  {
-    if (!this->subscriber->recv(&msg, 0))
-      return;
-    topic = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
-
-    // ToDo(caguero): Use this as extra metadata for the subscriber.
-    if (!this->subscriber->recv(&msg, 0))
-      return;
-    // sender = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
-
-    if (!this->subscriber->recv(&msg, 0))
-      return;
-    data = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
-  }
-  catch(const zmq::error_t &_error)
-  {
-    std::cout << "Error: " << _error.what() << std::endl;
-    return;
-  }
-
-  // Execute the callbacks registered.
+  std::string msgType;
   std::map<std::string, ISubscriptionHandler_M> handlers;
-  if (this->localSubscriptions.GetHandlers(topic, handlers))
+  ISubscriptionHandlerPtr firstSubscriberPtr;
+  bool handlersFound;
+  bool firstHandlerFound;
+
   {
-    // Get the first handler.
-    ISubscriptionHandlerPtr firstSubscriberPtr;
-    if (!this->localSubscriptions.GetHandler(topic, firstSubscriberPtr))
+    std::lock_guard<std::recursive_mutex> lock(this->mutex);
+
+    try
     {
-      std::cerr << "I couldn't find a subscriber. This should never happen."
-                << std::endl;
+      if (!this->subscriber->recv(&msg, 0))
+        return;
+      topic = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      // ToDo(caguero): Use this as extra metadata for the subscriber.
+      if (!this->subscriber->recv(&msg, 0))
+        return;
+      // sender = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      if (!this->subscriber->recv(&msg, 0))
+        return;
+      data = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      if (!this->subscriber->recv(&msg, 0))
+        return;
+      msgType = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+    }
+    catch(const zmq::error_t &_error)
+    {
+      std::cerr << "Error: " << _error.what() << std::endl;
       return;
     }
 
+    handlersFound = this->localSubscriptions.Handlers(topic, handlers);
+    firstHandlerFound = this->localSubscriptions.FirstHandler(topic, msgType,
+      firstSubscriberPtr);
+  }
+
+  // Execute the callbacks registered.
+  if (handlersFound && firstHandlerFound)
+  {
     // Create the message.
     auto recvMsg = firstSubscriberPtr->CreateMsg(data);
 
@@ -324,27 +362,30 @@ void NodeShared::RecvMsgUpdate()
       {
         ISubscriptionHandlerPtr subscriptionHandlerPtr = handler.second;
         if (subscriptionHandlerPtr)
-          subscriptionHandlerPtr->RunLocalCallback(topic, *recvMsg);
+        {
+          if (subscriptionHandlerPtr->TypeName() == msgType)
+            subscriptionHandlerPtr->RunLocalCallback(*recvMsg);
+        }
         else
           std::cerr << "Subscription handler is NULL" << std::endl;
       }
     }
   }
   else
-    std::cerr << "I am not subscribed to topic [" << topic << "]\n";
+    std::cerr << "I am not subscribed to topic [" << topic << "]" << std::endl;
 }
 
 //////////////////////////////////////////////////
 void NodeShared::RecvControlUpdate()
 {
-  std::lock_guard<std::recursive_mutex> lock(this->mutex);
-
   zmq::message_t msg(0);
   std::string topic;
   std::string procUuid;
   std::string nodeUuid;
   std::string data;
 
+  std::lock_guard<std::recursive_mutex> lock(this->mutex);
+
   try
   {
     if (!this->control->recv(&msg, 0))
@@ -375,12 +416,12 @@ void NodeShared::RecvControlUpdate()
     if (this->verbose)
     {
       std::cout << "Registering a new remote connection" << std::endl;
-      std::cout << "\tProc UUID: [" << procUuid << "]\n";
-      std::cout << "\tNode UUID: [" << nodeUuid << "]\n";
+      std::cout << "\tProc UUID: [" << procUuid << "]" << std::endl;
+      std::cout << "\tNode UUID: [" << nodeUuid << "]" << std::endl;
     }
 
     // Register that we have another remote subscriber.
-    MessagePublisher remoteNode(topic, "", "", procUuid, nodeUuid, Scope_t::All,
+    MessagePublisher remoteNode(topic, "", "", procUuid, nodeUuid, Scope_t::ALL,
       "");
     this->remoteSubscribers.AddPublisher(remoteNode);
   }
@@ -390,7 +431,7 @@ void NodeShared::RecvControlUpdate()
     {
       std::cout << "Registering the end of a remote connection" << std::endl;
       std::cout << "\tProc UUID: " << procUuid << std::endl;
-      std::cout << "\tNode UUID: [" << nodeUuid << "]\n";
+      std::cout << "\tNode UUID: [" << nodeUuid << "]" << std::endl;
     }
 
     // Delete a remote subscriber.
@@ -401,8 +442,6 @@ void NodeShared::RecvControlUpdate()
 //////////////////////////////////////////////////
 void NodeShared::RecvSrvRequest()
 {
-  std::lock_guard<std::recursive_mutex> lock(this->mutex);
-
   if (verbose)
     std::cout << "Message received requesting a service call" << std::endl;
 
@@ -415,74 +454,97 @@ void NodeShared::RecvSrvRequest()
   std::string rep;
   std::string resultStr;
   std::string dstId;
+  std::string reqType;
+  std::string repType;
+
+  IRepHandlerPtr repHandler;
+  bool hasHandler;
 
-  try
   {
-    if (!this->replier->recv(&msg, 0))
-      return;
+    std::lock_guard<std::recursive_mutex> lock(this->mutex);
 
-    if (!this->replier->recv(&msg, 0))
-      return;
-    topic = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+    try
+    {
+      if (!this->replier->recv(&msg, 0))
+        return;
 
-    if (!this->replier->recv(&msg, 0))
-      return;
-    sender = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+      if (!this->replier->recv(&msg, 0))
+        return;
+      topic = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
 
-    if (!this->replier->recv(&msg, 0))
-      return;
-    dstId = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+      if (!this->replier->recv(&msg, 0))
+        return;
+      sender = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
 
-    if (!this->replier->recv(&msg, 0))
-      return;
-    nodeUuid = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+      if (!this->replier->recv(&msg, 0))
+        return;
+      dstId = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
 
-    if (!this->replier->recv(&msg, 0))
-      return;
-    reqUuid = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+      if (!this->replier->recv(&msg, 0))
+        return;
+      nodeUuid = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
 
-    if (!this->replier->recv(&msg, 0))
+      if (!this->replier->recv(&msg, 0))
+        return;
+      reqUuid = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      if (!this->replier->recv(&msg, 0))
+        return;
+      req = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      if (!this->replier->recv(&msg, 0))
+        return;
+      reqType = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      if (!this->replier->recv(&msg, 0))
+        return;
+      repType = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+    }
+    catch(const zmq::error_t &_error)
+    {
+      std::cerr << "NodeShared::RecvSrvRequest() error parsing request: "
+                << _error.what() << std::endl;
       return;
-    req = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
-  }
-  catch(const zmq::error_t &_error)
-  {
-    std::cerr << "NodeShared::RecvSrvRequest() error parsing request: "
-              << _error.what() << std::endl;
-    return;
+    }
+
+    hasHandler =
+      this->repliers.FirstHandler(topic, reqType, repType, repHandler);
   }
 
   // Get the REP handler.
-  IRepHandlerPtr repHandler;
-  if (this->repliers.GetHandler(topic, repHandler))
+  if (hasHandler)
   {
     bool result;
     // Run the service call and get the results.
-    repHandler->RunCallback(topic, req, rep, result);
+    repHandler->RunCallback(req, rep, result);
 
     if (result)
       resultStr = "1";
     else
       resultStr = "0";
 
-    // I am still not connected to this address.
-    if (std::find(this->srvConnections.begin(), this->srvConnections.end(),
-          sender) == this->srvConnections.end())
     {
-      this->replier->connect(sender.c_str());
-      this->srvConnections.push_back(sender);
-      std::this_thread::sleep_for(std::chrono::milliseconds(100));
-
-      if (this->verbose)
+      std::lock_guard<std::recursive_mutex> lock(this->mutex);
+      // I am still not connected to this address.
+      if (std::find(this->srvConnections.begin(), this->srvConnections.end(),
+            sender) == this->srvConnections.end())
       {
-        std::cout << "\t* Connected to [" << sender
-                  << "] for sending a response" << std::endl;
+        this->replier->connect(sender.c_str());
+        this->srvConnections.push_back(sender);
+        std::this_thread::sleep_for(std::chrono::milliseconds(100));
+
+        if (this->verbose)
+        {
+          std::cout << "\t* Connected to [" << sender
+                    << "] for sending a response" << std::endl;
+        }
       }
     }
 
     // Send the reply.
     try
     {
+      std::lock_guard<std::recursive_mutex> lock(this->mutex);
       zmq::message_t response;
 
       response.rebuild(dstId.size());
@@ -517,15 +579,13 @@ void NodeShared::RecvSrvRequest()
     }
   }
   // else
-  //  std::cerr << "I do not have a service call registered for topic ["
-  //            << topic << "]\n";
+  //   std::cerr << "I do not have a service call registered for topic ["
+  //             << topic << "]\n";
 }
 
 //////////////////////////////////////////////////
 void NodeShared::RecvSrvResponse()
 {
-  std::lock_guard<std::recursive_mutex> lock(this->mutex);
-
   if (verbose)
     std::cout << "Message received containing a service call REP" << std::endl;
 
@@ -537,50 +597,62 @@ void NodeShared::RecvSrvResponse()
   std::string resultStr;
   bool result;
 
-  try
-  {
-    if (!this->responseReceiver->recv(&msg, 0))
-      return;
-
-    if (!this->responseReceiver->recv(&msg, 0))
-      return;
-    topic = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
-
-    if (!this->responseReceiver->recv(&msg, 0))
-      return;
-    nodeUuid = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
-
-    if (!this->responseReceiver->recv(&msg, 0))
-      return;
-    reqUuid = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
-
-    if (!this->responseReceiver->recv(&msg, 0))
-      return;
-    rep = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
-
-    if (!this->responseReceiver->recv(&msg, 0))
-      return;
-    resultStr = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
-    result = resultStr == "1";
-  }
-  catch(const zmq::error_t &_error)
-  {
-    std::cerr << "NodeShared::RecvSrvResponse() error: "
-              << _error.what() << std::endl;
-    return;
-  }
-
   IReqHandlerPtr reqHandlerPtr;
-  if (this->requests.GetHandler(topic, nodeUuid, reqUuid, reqHandlerPtr))
+  bool hasHandler;
+
+  {
+    std::lock_guard<std::recursive_mutex> lock(this->mutex);
+
+    try
+    {
+      if (!this->responseReceiver->recv(&msg, 0))
+        return;
+
+      if (!this->responseReceiver->recv(&msg, 0))
+        return;
+      topic = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      if (!this->responseReceiver->recv(&msg, 0))
+        return;
+      nodeUuid = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      if (!this->responseReceiver->recv(&msg, 0))
+        return;
+      reqUuid = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      if (!this->responseReceiver->recv(&msg, 0))
+        return;
+      rep = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+
+      if (!this->responseReceiver->recv(&msg, 0))
+        return;
+      resultStr = std::string(reinterpret_cast<char *>(msg.data()), msg.size());
+      result = resultStr == "1";
+    }
+    catch(const zmq::error_t &_error)
+    {
+      std::cerr << "NodeShared::RecvSrvResponse() error: "
+                << _error.what() << std::endl;
+      return;
+    }
+
+    hasHandler =
+      this->requests.Handler(topic, nodeUuid, reqUuid, reqHandlerPtr);
+  }
+
+  if (hasHandler)
   {
     // Notify the result.
-    reqHandlerPtr->NotifyResult(topic, rep, result);
+    reqHandlerPtr->NotifyResult(rep, result);
 
     // Remove the handler.
-    if (!this->requests.RemoveHandler(topic, nodeUuid, reqUuid))
+    std::lock_guard<std::recursive_mutex> lock(this->mutex);
     {
-      std::cerr << "NodeShare::RecvSrvResponse(): "
-                << "Error removing request handler" << std::endl;
+      if (!this->requests.RemoveHandler(topic, nodeUuid, reqUuid))
+      {
+        std::cerr << "NodeShare::RecvSrvResponse(): "
+                  << "Error removing request handler" << std::endl;
+      }
     }
   }
   else
@@ -591,7 +663,8 @@ void NodeShared::RecvSrvResponse()
 }
 
 //////////////////////////////////////////////////
-void NodeShared::SendPendingRemoteReqs(const std::string &_topic)
+void NodeShared::SendPendingRemoteReqs(const std::string &_topic,
+  const std::string &_reqType, const std::string &_repType)
 {
   std::string responserAddr;
   std::string responserId;
@@ -600,10 +673,28 @@ void NodeShared::SendPendingRemoteReqs(const std::string &_topic)
   if (addresses.empty())
     return;
 
-  // Get the first responder.
-  auto &v = addresses.begin()->second;
-  responserAddr = v.at(0).Addr();
-  responserId = v.at(0).SocketId();
+  // Find a publisher that offers this service with a particular pair of REQ/REP
+  // types.
+  bool found = false;
+  for (auto &proc : addresses)
+  {
+    auto &v = proc.second;
+    for (auto &pub : v)
+    {
+      if (pub.ReqTypeName() == _reqType && pub.RepTypeName() == _repType)
+      {
+        found = true;
+        responserAddr = pub.Addr();
+        responserId = pub.SocketId();
+        break;
+      }
+    }
+    if (found)
+      break;
+  }
+
+  if (!found)
+    return;
 
   if (verbose)
   {
@@ -611,9 +702,25 @@ void NodeShared::SendPendingRemoteReqs(const std::string &_topic)
               << responserAddr << "]" << std::endl;
   }
 
+  std::lock_guard<std::recursive_mutex> lock(this->mutex);
+
+  // I am still not connected to this address.
+  if (std::find(this->srvConnections.begin(), this->srvConnections.end(),
+        responserAddr) == this->srvConnections.end())
+  {
+    this->requester->connect(responserAddr.c_str());
+    this->srvConnections.push_back(responserAddr);
+    std::this_thread::sleep_for(std::chrono::milliseconds(100));
+    if (this->verbose)
+    {
+      std::cout << "\t* Connected to [" << responserAddr
+                << "] for service requests" << std::endl;
+    }
+  }
+
   // Send all the pending REQs.
   IReqHandler_M reqs;
-  if (!this->requests.GetHandlers(_topic, reqs))
+  if (!this->requests.Handlers(_topic, reqs))
     return;
 
   for (auto &node : reqs)
@@ -624,6 +731,13 @@ void NodeShared::SendPendingRemoteReqs(const std::string &_topic)
       if (req.second->Requested())
         continue;
 
+      // Check that the pending service call has types that match the responser.
+      if (req.second->ReqTypeName() != _reqType ||
+          req.second->RepTypeName() != _repType)
+      {
+        continue;
+      }
+
       // Mark the handler as requested.
       req.second->Requested(true);
 
@@ -666,9 +780,17 @@ void NodeShared::SendPendingRemoteReqs(const std::string &_topic)
 
         msg.rebuild(data.size());
         memcpy(msg.data(), data.data(), data.size());
+        this->requester->send(msg, ZMQ_SNDMORE);
+
+        msg.rebuild(_reqType.size());
+        memcpy(msg.data(), _reqType.data(), _reqType.size());
+        this->requester->send(msg, ZMQ_SNDMORE);
+
+        msg.rebuild(_repType.size());
+        memcpy(msg.data(), _repType.data(), _repType.size());
         this->requester->send(msg, 0);
       }
-      catch(const zmq::error_t& ze)
+      catch(const zmq::error_t& /*ze*/)
       {
         // Debug output.
         // std::cerr << "Error connecting [" << ze.what() << "]\n";
@@ -723,15 +845,18 @@ void NodeShared::OnNewConnection(const MessagePublisher &_pub)
       socket.setsockopt(ZMQ_LINGER, &lingerVal, sizeof(lingerVal));
       socket.connect(ctrl.c_str());
 
-      std::this_thread::sleep_for(std::chrono::milliseconds(300));
+      std::this_thread::sleep_for(std::chrono::milliseconds(100));
 
       std::map<std::string, ISubscriptionHandler_M> handlers;
-      if (this->localSubscriptions.GetHandlers(topic, handlers))
+      if (this->localSubscriptions.Handlers(topic, handlers))
       {
         for (auto &node : handlers)
         {
           for (auto &handler : node.second)
           {
+            if (handler.second->TypeName() != _pub.MsgTypeName())
+              continue;
+
             std::string nodeUuid = handler.second->NodeUuid();
 
             zmq::message_t msg;
@@ -756,7 +881,7 @@ void NodeShared::OnNewConnection(const MessagePublisher &_pub)
       }
     }
     // The remote node might not be available when we are connecting.
-    catch(const zmq::error_t& ze)
+    catch(const zmq::error_t& /*ze*/)
     {
     }
   }
@@ -783,14 +908,9 @@ void NodeShared::OnNewDisconnection(const MessagePublisher &_pub)
     this->remoteSubscribers.DelPublisherByNode(topic, procUuid, nUuid);
 
     MessagePublisher connection;
-    if (!this->connections.GetPublisher(topic, procUuid, nUuid, connection))
+    if (!this->connections.Publisher(topic, procUuid, nUuid, connection))
       return;
 
-    // Disconnect from a publisher's socket.
-    // for (const auto &connection : this->connections[procUuid])
-    //   this->subscriber->disconnect(connection.addr.c_str());
-    this->subscriber->disconnect(connection.Addr().c_str());
-
     // I am no longer connected.
     this->connections.DelPublisherByNode(topic, procUuid, nUuid);
   }
@@ -799,13 +919,9 @@ void NodeShared::OnNewDisconnection(const MessagePublisher &_pub)
     this->remoteSubscribers.DelPublishersByProc(procUuid);
 
     MsgAddresses_M info;
-    if (!this->connections.GetPublishers(topic, info))
+    if (!this->connections.Publishers(topic, info))
       return;
 
-    // Disconnect from all the connections of that publisher.
-    for (auto &connection : info[procUuid])
-      this->subscriber->disconnect(connection.Addr().c_str());
-
     // Remove all the connections from the process disonnected.
     this->connections.DelPublishersByProc(procUuid);
   }
@@ -816,6 +932,8 @@ void NodeShared::OnNewSrvConnection(const ServicePublisher &_pub)
 {
   std::string topic = _pub.Topic();
   std::string addr = _pub.Addr();
+  std::string reqType = _pub.ReqTypeName();
+  std::string repType = _pub.RepTypeName();
 
   std::lock_guard<std::recursive_mutex> lock(this->mutex);
 
@@ -831,7 +949,7 @@ void NodeShared::OnNewSrvConnection(const ServicePublisher &_pub)
   {
     this->requester->connect(addr.c_str());
     this->srvConnections.push_back(addr);
-    std::this_thread::sleep_for(std::chrono::milliseconds(300));
+    std::this_thread::sleep_for(std::chrono::milliseconds(100));
     if (this->verbose)
     {
       std::cout << "\t* Connected to [" << addr
@@ -839,8 +957,14 @@ void NodeShared::OnNewSrvConnection(const ServicePublisher &_pub)
     }
   }
 
-  // Request all pending service calls for this topic.
-  this->SendPendingRemoteReqs(topic);
+  // Check if there's a pending service request with this specific combination
+  // of request and response types.
+  IReqHandlerPtr handler;
+  if (this->requests.FirstHandler(topic, reqType, repType, handler))
+  {
+    // Request all pending service calls for this topic and req/rep types.
+    this->SendPendingRemoteReqs(topic, reqType, repType);
+  }
 }
 
 //////////////////////////////////////////////////
