@@ -18,14 +18,19 @@
 #include <chrono>
 #include <csignal>
 #include <cstdlib>
+#include <functional>
 #include <memory>
 #include <string>
 #include <thread>
+
 #include "gtest/gtest.h"
+#include "ignition/transport/AdvertiseOptions.hh"
 #include "ignition/transport/Node.hh"
+#include "ignition/transport/NodeOptions.hh"
 #include "ignition/transport/TopicUtils.hh"
 #include "ignition/transport/test_config.h"
 #include "msgs/int.pb.h"
+#include "msgs/vector3d.pb.h"
 
 using namespace ignition;
 
@@ -36,8 +41,10 @@ std::mutex exitMutex;
 int data = 5;
 bool cbExecuted;
 bool cb2Executed;
+bool cbVectorExecuted;
 bool srvExecuted;
 bool responseExecuted;
+bool wrongResponseExecuted;
 int counter = 0;
 bool terminatePub = false;
 
@@ -48,16 +55,17 @@ void reset()
   cbExecuted = false;
   cb2Executed = false;
   srvExecuted = false;
+  cbVectorExecuted = false;
   responseExecuted = false;
+  wrongResponseExecuted = false;
   counter = 0;
   terminatePub = false;
 }
 
 //////////////////////////////////////////////////
 /// \brief Function called each time a topic update is received.
-void cb(const std::string &_topic, const transport::msgs::Int &_msg)
+void cb(const transport::msgs::Int &_msg)
 {
-  EXPECT_EQ(_topic, topic);
   EXPECT_EQ(_msg.data(), data);
   cbExecuted = true;
   counter++;
@@ -65,19 +73,17 @@ void cb(const std::string &_topic, const transport::msgs::Int &_msg)
 
 //////////////////////////////////////////////////
 /// \brief Function called each time a topic update is received.
-void cb2(const std::string &_topic, const transport::msgs::Int &_msg)
+void cb2(const transport::msgs::Int &_msg)
 {
-  EXPECT_EQ(_topic, topic);
   EXPECT_EQ(_msg.data(), data);
   cb2Executed = true;
 }
 
 //////////////////////////////////////////////////
 /// \brief Provide a service call.
-void srvEcho(const std::string &_topic, const transport::msgs::Int &_req,
+void srvEcho(const transport::msgs::Int &_req,
   transport::msgs::Int &_rep, bool &_result)
 {
-  EXPECT_EQ(_topic, topic);
   srvExecuted = true;
 
   EXPECT_EQ(_req.data(), data);
@@ -87,10 +93,8 @@ void srvEcho(const std::string &_topic, const transport::msgs::Int &_req,
 
 //////////////////////////////////////////////////
 /// \brief Service call response callback.
-void response(const std::string &_topic, const transport::msgs::Int &_rep,
-  bool _result)
+void response(const transport::msgs::Int &_rep, const bool _result)
 {
-  EXPECT_EQ(_topic, topic);
   EXPECT_EQ(_rep.data(), data);
   EXPECT_TRUE(_result);
 
@@ -98,6 +102,20 @@ void response(const std::string &_topic, const transport::msgs::Int &_rep,
   ++counter;
 }
 
+//////////////////////////////////////////////////
+/// \brief Service call response callback.
+void wrongResponse(const transport::msgs::Vector3d &/*_rep*/, bool /*_result*/)
+{
+  wrongResponseExecuted = true;
+}
+
+//////////////////////////////////////////////////
+/// \brief Callback for receiving Vector3d data.
+void cbVector(const transport::msgs::Vector3d &/*_msg*/)
+{
+  cbVectorExecuted = true;
+}
+
 //////////////////////////////////////////////////
 /// \brief A class for testing subscription passing a member function
 /// as a callback.
@@ -106,7 +124,8 @@ class MyTestClass
   /// \brief Class constructor.
   public: MyTestClass()
     : callbackExecuted(false),
-      callbackSrvExecuted(false)
+      callbackSrvExecuted(false),
+      wrongCallbackSrvExecuted(false)
   {
     // Subscribe to an illegal topic.
     EXPECT_FALSE(this->node.Subscribe("Bad Topic", &MyTestClass::Cb, this));
@@ -115,22 +134,36 @@ class MyTestClass
   }
 
   // Member function used as a callback for responding to a service call.
-  public: void Echo(const std::string &_topic,
-    const transport::msgs::Int &_req, transport::msgs::Int &_rep,
-    bool &_result)
+  public: void Echo(const transport::msgs::Int &_req,
+    transport::msgs::Int &_rep, bool &_result)
   {
-    EXPECT_EQ(_topic, topic);
     EXPECT_EQ(_req.data(), data);
     _rep.set_data(_req.data());
     _result = true;
     this->callbackSrvExecuted = true;
   }
 
+  /// \brief Member function used as a callback for responding to a service call
+  public: void WrongEcho(const transport::msgs::Vector3d &/*_req*/,
+    transport::msgs::Int &/*_rep*/, bool &_result)
+  {
+    _result = true;
+    this->wrongCallbackSrvExecuted = true;
+  }
+
+  /// \brief Response callback to a service request.
+  public: void EchoResponse(const transport::msgs::Int &_rep,
+    const bool _result)
+  {
+    EXPECT_EQ(_rep.data(), data);
+    EXPECT_TRUE(_result);
+
+    this->responseExecuted = true;
+  }
+
   /// \brief Member function called each time a topic update is received.
-  public: void Cb(const std::string &_topic,
-    const transport::msgs::Int &_msg)
+  public: void Cb(const transport::msgs::Int &_msg)
   {
-    EXPECT_EQ(_topic, topic);
     EXPECT_EQ(_msg.data(), data);
     this->callbackExecuted = true;
   };
@@ -142,9 +175,9 @@ class MyTestClass
     msg.set_data(data);
 
     // Advertise an illegal topic.
-    EXPECT_FALSE(this->node.Advertise("invalid topic"));
+    EXPECT_FALSE(this->node.Advertise<transport::msgs::Int>("invalid topic"));
 
-    EXPECT_TRUE(this->node.Advertise(topic));
+    EXPECT_TRUE(this->node.Advertise<transport::msgs::Int>(topic));
     EXPECT_TRUE(this->node.Publish(topic, msg));
   }
 
@@ -152,23 +185,56 @@ class MyTestClass
   {
     transport::msgs::Int req;
     transport::msgs::Int rep;
+    transport::msgs::Vector3d wrongReq;
+    transport::msgs::Vector3d wrongRep;
     int timeout = 500;
     bool result;
 
     req.set_data(data);
 
+    this->Reset();
+
     // Advertise an illegal service name.
     EXPECT_FALSE(this->node.Advertise("Bad Srv", &MyTestClass::Echo, this));
 
+    // Advertise and request a valid service.
     EXPECT_TRUE(this->node.Advertise(topic, &MyTestClass::Echo, this));
     EXPECT_TRUE(this->node.Request(topic, req, timeout, rep, result));
     ASSERT_TRUE(result);
     EXPECT_EQ(rep.data(), data);
+    EXPECT_TRUE(this->callbackSrvExecuted);
+
+    this->Reset();
+
+    // Request a valid service using a member function callback.
+    this->node.Request(topic, req, &MyTestClass::EchoResponse, this);
+    EXPECT_TRUE(this->responseExecuted);
+
+    this->Reset();
+
+    // Service requests with wrong types.
+    EXPECT_FALSE(this->node.Request(topic, wrongReq, timeout, rep, result));
+    EXPECT_FALSE(this->node.Request(topic, req, timeout, wrongRep, result));
+    EXPECT_TRUE(this->node.Request(topic, wrongReq, response));
+    EXPECT_TRUE(this->node.Request(topic, req, wrongResponse));
+    std::this_thread::sleep_for(std::chrono::milliseconds(300));
+    EXPECT_FALSE(this->callbackSrvExecuted);
+    EXPECT_FALSE(this->wrongCallbackSrvExecuted);
   }
 
-  /// \brief Member variable that flags when the callback is executed.
+  public: void Reset()
+  {
+    this->callbackExecuted = false;
+    this->callbackSrvExecuted = false;
+    this->wrongCallbackSrvExecuted = false;
+    this->responseExecuted = false;
+  }
+
+  /// \brief Member variables that flag when the callbacks are executed.
   public: bool callbackExecuted;
   public: bool callbackSrvExecuted;
+  public: bool wrongCallbackSrvExecuted;
+  public: bool responseExecuted;
 
   /// \brief Transport node;
   private: transport::Node node;
@@ -194,15 +260,18 @@ void CreateSubscriber()
 /// will publish a message, whereas the other thread is subscribed to the topic.
 /// \param[in] _scope Scope used to advertise the topic.
 void CreatePubSubTwoThreads(
-  const transport::Scope_t &_sc = transport::Scope_t::All)
+  const transport::Scope_t &_sc = transport::Scope_t::ALL)
 {
   reset();
 
+  transport::AdvertiseOptions opts;
+  opts.SetScope(_sc);
+
   transport::msgs::Int msg;
   msg.set_data(data);
 
   transport::Node node;
-  EXPECT_TRUE(node.Advertise(topic, _sc));
+  EXPECT_TRUE(node.Advertise<transport::msgs::Int>(topic, opts));
 
   // Subscribe to a topic in a different thread and wait until the callback is
   // received.
@@ -232,8 +301,13 @@ TEST(NodeTest, PubWithoutAdvertise)
 
   // Check that an invalid namespace is ignored. The callbacks are expecting an
   // empty namespace.
-  transport::Node node1(partition, "invalid namespace");
-  transport::Node node2(partition, "");
+  transport::NodeOptions optionsNode1;
+  transport::NodeOptions optionsNode2;
+  optionsNode1.SetPartition(partition);
+  optionsNode1.SetNameSpace("invalid namespace");
+  optionsNode2.SetPartition(partition);
+  transport::Node node1(optionsNode1);
+  transport::Node node2(optionsNode2);
 
   // Check the advertised/subscribed topics and advertised services.
   EXPECT_TRUE(node1.AdvertisedTopics().empty());
@@ -243,13 +317,13 @@ TEST(NodeTest, PubWithoutAdvertise)
   // Publish some data on topic without advertising it first.
   EXPECT_FALSE(node1.Publish(topic, msg));
 
-  EXPECT_TRUE(node1.Advertise(topic));
+  EXPECT_TRUE(node1.Advertise<transport::msgs::Int>(topic));
 
   auto advertisedTopics = node1.AdvertisedTopics();
   ASSERT_EQ(advertisedTopics.size(), 1u);
   EXPECT_EQ(advertisedTopics.at(0), topic);
 
-  EXPECT_TRUE(node2.Advertise(topic));
+  EXPECT_TRUE(node2.Advertise<transport::msgs::Int>(topic));
   advertisedTopics = node2.AdvertisedTopics();
   ASSERT_EQ(advertisedTopics.size(), 1u);
   EXPECT_EQ(advertisedTopics.at(0), topic);
@@ -286,9 +360,9 @@ TEST(NodeTest, PubSubSameThread)
   transport::Node node;
 
   // Advertise an illegal topic.
-  EXPECT_FALSE(node.Advertise("invalid topic"));
+  EXPECT_FALSE(node.Advertise<transport::msgs::Int>("invalid topic"));
 
-  EXPECT_TRUE(node.Advertise(topic));
+  EXPECT_TRUE(node.Advertise<transport::msgs::Int>(topic));
 
   // Subscribe to an illegal topic.
   EXPECT_FALSE(node.Subscribe("invalid topic", cb));
@@ -337,6 +411,36 @@ TEST(NodeTest, PubSubSameThread)
   EXPECT_FALSE(cbExecuted);
 }
 
+//////////////////////////////////////////////////
+/// \brief Subscribe to a topic using a lambda function.
+TEST(NodeTest, PubSubSameThreadLamda)
+{
+  transport::msgs::Int msg;
+  msg.set_data(data);
+
+  transport::Node node;
+
+  EXPECT_TRUE(node.Advertise<transport::msgs::Int>(topic));
+
+  bool executed = false;
+  std::function<void(const transport::msgs::Int&)> subCb =
+    [&executed](const transport::msgs::Int &_msg)
+  {
+    EXPECT_EQ(_msg.data(), data);
+    executed = true;
+  };
+
+  EXPECT_TRUE(node.Subscribe(topic, subCb));
+
+  // Give some time to the subscribers.
+  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+
+  // Publish a first message.
+  EXPECT_TRUE(node.Publish(topic, msg));
+
+  EXPECT_TRUE(executed);
+}
+
 //////////////////////////////////////////////////
 /// \brief Use two threads using their own transport nodes. One thread
 /// will publish a message, whereas the other thread is subscribed to the topic.
@@ -359,7 +463,7 @@ TEST(NodeTest, PubSubOneThreadTwoSubs)
   transport::Node node1;
   transport::Node node2;
 
-  EXPECT_TRUE(node1.Advertise(topic));
+  EXPECT_TRUE(node1.Advertise<transport::msgs::Int>(topic));
 
   // Subscribe to topic in node1.
   EXPECT_TRUE(node1.Subscribe(topic, cb));
@@ -443,8 +547,6 @@ TEST(NodeTest, ClassMemberCallback)
   EXPECT_TRUE(client.callbackExecuted);
 
   client.TestServiceCall();
-
-  EXPECT_TRUE(client.callbackSrvExecuted);
 }
 
 //////////////////////////////////////////////////
@@ -452,23 +554,42 @@ TEST(NodeTest, ClassMemberCallback)
 /// advertising a topic with "Process" scope.
 TEST(NodeTest, ScopeProcess)
 {
-  CreatePubSubTwoThreads(transport::Scope_t::Process);
+  CreatePubSubTwoThreads(transport::Scope_t::PROCESS);
 }
 
 //////////////////////////////////////////////////
-/// \brief Check that two nodes in diffetent threads are able to communicate
+/// \brief Check that two nodes in different threads are able to communicate
 /// advertising a topic with "Host" scope.
 TEST(NodeTest, ScopeHost)
 {
-  CreatePubSubTwoThreads(transport::Scope_t::Host);
+  CreatePubSubTwoThreads(transport::Scope_t::HOST);
 }
 
 //////////////////////////////////////////////////
-/// \brief Check that two nodes in diffetent threads are able to communicate
+/// \brief Check that two nodes in different threads are able to communicate
 /// advertising a topic with "All" scope.
 TEST(NodeTest, ScopeAll)
 {
-  CreatePubSubTwoThreads(transport::Scope_t::All);
+  CreatePubSubTwoThreads(transport::Scope_t::ALL);
+}
+
+//////////////////////////////////////////////////
+/// \brief Check that the types advertised and published match.
+TEST(NodeTest, TypeMismatch)
+{
+  transport::msgs::Int rightMsg;
+  transport::msgs::Vector3d wrongMsg;
+  rightMsg.set_data(1);
+  wrongMsg.set_x(1);
+  wrongMsg.set_y(2);
+  wrongMsg.set_z(3);
+
+  transport::Node node;
+
+  EXPECT_TRUE(node.Advertise<transport::msgs::Int>(topic));
+
+  EXPECT_FALSE(node.Publish(topic, wrongMsg));
+  EXPECT_TRUE(node.Publish(topic, rightMsg));
 }
 
 //////////////////////////////////////////////////
@@ -535,6 +656,40 @@ TEST(NodeTest, ServiceCallAsync)
   ASSERT_TRUE(node.AdvertisedServices().empty());
 }
 
+//////////////////////////////////////////////////
+/// \brief Make an asynchronous service call using lambdas.
+TEST(NodeTest, ServiceCallAsyncLambda)
+{
+  std::function<void(const transport::msgs::Int &, transport::msgs::Int &,
+    bool &)> advCb = [](const transport::msgs::Int &_req,
+      transport::msgs::Int &_rep, bool &_result)
+  {
+    EXPECT_EQ(_req.data(), data);
+    _rep.set_data(_req.data());
+    _result = true;
+  };
+
+  transport::Node node;
+  EXPECT_TRUE((node.Advertise<transport::msgs::Int, transport::msgs::Int>(topic,
+    advCb)));
+
+  bool executed = false;
+  std::function<void(const transport::msgs::Int &, const bool)> reqCb =
+    [&executed](const transport::msgs::Int &_rep, const bool _result)
+  {
+    EXPECT_EQ(_rep.data(), data);
+    EXPECT_TRUE(_result);
+    executed = true;
+  };
+
+  transport::msgs::Int req;
+  req.set_data(data);
+
+  EXPECT_TRUE((node.Request(topic, req, reqCb)));
+
+  EXPECT_TRUE(executed);
+}
+
 //////////////////////////////////////////////////
 /// \brief Request multiple service calls at the same time.
 TEST(NodeTest, MultipleServiceCallAsync)
@@ -626,21 +781,23 @@ TEST(NodeTest, ServiceCallSyncTimeout)
   transport::msgs::Int req;
   transport::msgs::Int rep;
   bool result;
-  unsigned int timeout = 1000;
+  int64_t timeout = 1000;
 
   req.set_data(data);
 
   transport::Node node;
 
   auto t1 = std::chrono::system_clock::now();
-  bool executed = node.Request(topic, req, timeout, rep, result);
+  bool executed = node.Request(topic, req, static_cast<unsigned int>(timeout),
+      rep, result);
   auto t2 = std::chrono::system_clock::now();
 
-  double elapsed =
+  int64_t elapsed =
     std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
 
   // Check if the elapsed time was close to the timeout.
-  EXPECT_NEAR(elapsed, timeout, 10.0);
+  auto diff = std::max(elapsed, timeout) - std::min(elapsed, timeout);
+  EXPECT_LE(diff, 10);
 
   // Check that the service call response was not executed.
   EXPECT_FALSE(executed);
@@ -657,10 +814,11 @@ void createInfinitePublisher()
   msg.set_data(data);
   transport::Node node;
 
-  EXPECT_TRUE(node.Advertise(topic));
+  EXPECT_TRUE(node.Advertise<transport::msgs::Int>(topic));
 
   auto i = 0;
-  while (true)
+  bool exitLoop = false;
+  while (!exitLoop)
   {
     EXPECT_TRUE(node.Publish(topic, msg));
     ++i;
@@ -669,7 +827,7 @@ void createInfinitePublisher()
     {
       std::lock_guard<std::mutex> lock(exitMutex);
       if (terminatePub)
-        break;
+        exitLoop = true;
     }
   }
 
@@ -739,6 +897,286 @@ TEST(NodeTest, SigTermTermination)
 #endif
 }
 
+//////////////////////////////////////////////////
+/// \brief Check that a message is not published if the type does not match
+/// the type advertised.
+TEST(NodeTest, PubSubWrongTypesOnPublish)
+{
+  reset();
+
+  transport::msgs::Int msg;
+  msg.set_data(data);
+  transport::msgs::Vector3d msgV;
+  msgV.set_x(1);
+  msgV.set_y(2);
+  msgV.set_z(3);
+
+  transport::Node node;
+
+  EXPECT_TRUE(node.Advertise<transport::msgs::Int>(topic));
+
+  EXPECT_TRUE(node.Subscribe(topic, cb));
+
+  // Wait some time before publishing.
+  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+
+  // Send a message with a wrong type.
+  EXPECT_FALSE(node.Publish(topic, msgV));
+
+  // Check that the message was not received.
+  EXPECT_FALSE(cbExecuted);
+
+  reset();
+
+  // Publish a second message on topic.
+  EXPECT_TRUE(node.Publish(topic, msg));
+
+  // Give some time to the subscribers.
+  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+
+  // Check that the data was received.
+  EXPECT_TRUE(cbExecuted);
+
+  reset();
+}
+
+//////////////////////////////////////////////////
+/// \brief Check that a message is not received if the callback does not use
+/// the advertised types.
+TEST(NodeTest, PubSubWrongTypesOnSubscription)
+{
+  reset();
+
+  transport::msgs::Vector3d msgV;
+  msgV.set_x(1);
+  msgV.set_y(2);
+  msgV.set_z(3);
+
+  transport::Node node;
+
+  EXPECT_TRUE(node.Advertise<transport::msgs::Vector3d>(topic));
+
+  EXPECT_TRUE(node.Subscribe(topic, cb));
+
+  // Wait some time before publishing.
+  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+
+  // Send a message with a wrong type.
+  EXPECT_TRUE(node.Publish(topic, msgV));
+
+  // Check that the message was not received.
+  EXPECT_FALSE(cbExecuted);
+
+  reset();
+}
+
+//////////////////////////////////////////////////
+/// \brief This test spawns two subscribers on the same topic. One of the
+/// subscribers has a wrong callback (types in the callback does not match the
+/// advertised type). Check that only the good callback is executed.
+TEST(NodeTest, PubSubWrongTypesTwoSubscribers)
+{
+  reset();
+
+  transport::msgs::Int msg;
+  msg.set_data(data);
+
+  transport::Node node1;
+  transport::Node node2;
+
+  EXPECT_TRUE(node1.Advertise<transport::msgs::Int>(topic));
+
+  // Good subscriber.
+  EXPECT_TRUE(node1.Subscribe(topic, cb));
+
+  // Bad subscriber: cbVector does not match the types advertised by node1.
+  EXPECT_TRUE(node2.Subscribe(topic, cbVector));
+
+  // Wait some time before publishing.
+  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+
+  EXPECT_TRUE(node1.Publish(topic, msg));
+
+  // Give some time to the subscribers.
+  std::this_thread::sleep_for(std::chrono::milliseconds(100));
+
+  // Check that the message was received by node1.
+  EXPECT_TRUE(cbExecuted);
+
+  // Check that the message was not received by node2.
+  EXPECT_FALSE(cbVectorExecuted);
+}
+
+//////////////////////////////////////////////////
+/// \brief This test spawns a service responser and a service requester. The
+/// requester uses a wrong type for the request argument. The test should verify
+/// that the service call does not succeed.
+TEST(NodeTest, SrvRequestWrongReq)
+{
+  transport::msgs::Vector3d wrongReq;
+  transport::msgs::Int rep;
+  bool result;
+  unsigned int timeout = 1000;
+
+  wrongReq.set_x(1);
+  wrongReq.set_y(2);
+  wrongReq.set_z(3);
+
+  reset();
+
+  transport::Node node;
+  EXPECT_TRUE(node.Advertise(topic, srvEcho));
+
+  // Request an asynchronous service call with wrong type in the request.
+  EXPECT_TRUE(node.Request(topic, wrongReq, response));
+  std::this_thread::sleep_for(std::chrono::milliseconds(300));
+  EXPECT_FALSE(responseExecuted);
+
+  // Request a synchronous service call with wrong type in the request.
+  EXPECT_FALSE(node.Request(topic, wrongReq, timeout, rep, result));
+
+  reset();
+}
+
+//////////////////////////////////////////////////
+/// \brief This test spawns a service responser and a service requester. The
+/// requester uses a wrong type for the response argument. The test should
+/// verify that the service call does not succeed.
+TEST(NodeTest, SrvRequestWrongRep)
+{
+  transport::msgs::Int req;
+  transport::msgs::Vector3d wrongRep;
+  bool result;
+  unsigned int timeout = 1000;
+
+  req.set_data(data);
+
+  reset();
+
+  transport::Node node;
+  EXPECT_TRUE(node.Advertise(topic, srvEcho));
+
+  // Request an asynchronous service call with wrong type in the response.
+  EXPECT_TRUE(node.Request(topic, req, wrongResponse));
+  std::this_thread::sleep_for(std::chrono::milliseconds(300));
+  EXPECT_FALSE(wrongResponseExecuted);
+
+  // Request a synchronous service call with wrong type in the response.
+  EXPECT_FALSE(node.Request(topic, req, timeout, wrongRep, result));
+
+  reset();
+}
+
+//////////////////////////////////////////////////
+/// \brief This test spawns a service responser and two service requesters. One
+/// requester uses wrong type arguments. The test should verify that only one
+/// of the requesters receives the response.
+TEST(NodeTest, SrvTwoRequestsOneWrong)
+{
+  transport::msgs::Int req;
+  transport::msgs::Int goodRep;
+  transport::msgs::Vector3d badRep;
+  bool result;
+  unsigned int timeout = 1000;
+
+  req.set_data(data);
+
+  transport::Node node;
+  EXPECT_TRUE(node.Advertise(topic, srvEcho));
+
+  // Request service calls with wrong types in the response.
+  EXPECT_FALSE(node.Request(topic, req, timeout, badRep, result));
+  EXPECT_TRUE(node.Request(topic, req, wrongResponse));
+  std::this_thread::sleep_for(std::chrono::milliseconds(300));
+  EXPECT_FALSE(wrongResponseExecuted);
+
+  // Valid service requests.
+  EXPECT_TRUE(node.Request(topic, req, timeout, goodRep, result));
+  EXPECT_TRUE(node.Request(topic, req, response));
+  std::this_thread::sleep_for(std::chrono::milliseconds(300));
+  EXPECT_TRUE(responseExecuted);
+}
+
+//////////////////////////////////////////////////
+/// \brief This test creates two nodes and advertises some topics. The test
+/// verifies that TopicList() returns the list of all the topics advertised.
+TEST(NodeTest, TopicList)
+{
+  std::vector<std::string> topics;
+  transport::Node node1;
+  transport::Node node2;
+
+  node1.Advertise<transport::msgs::Int>("topic1");
+  node2.Advertise<transport::msgs::Int>("topic2");
+
+  node1.TopicList(topics);
+  EXPECT_EQ(topics.size(), 2u);
+  topics.clear();
+
+  auto start = std::chrono::steady_clock::now();
+  node1.TopicList(topics);
+  auto end = std::chrono::steady_clock::now();
+  EXPECT_EQ(topics.size(), 2u);
+
+  // The first TopicList() call might block if the discovery is still
+  // initializing (it may happen if we run this test alone).
+  // However, the second call should never block.
+  auto elapsed = end - start;
+  EXPECT_LT(std::chrono::duration_cast<std::chrono::milliseconds>
+      (elapsed).count(), 2);
+}
+
+//////////////////////////////////////////////////
+/// \brief This test creates two nodes and advertises some services. The test
+/// verifies that ServiceList() returns the list of all the services advertised.
+TEST(NodeTest, ServiceList)
+{
+  std::vector<std::string> services;
+  transport::Node node;
+
+  node.Advertise(topic, srvEcho);
+
+  node.ServiceList(services);
+  EXPECT_EQ(services.size(), 1u);
+  services.clear();
+
+  auto start = std::chrono::steady_clock::now();
+  node.ServiceList(services);
+  auto end = std::chrono::steady_clock::now();
+  EXPECT_EQ(services.size(), 1u);
+
+  // The first TopicList() call might block if the discovery is still
+  // initializing (it may happen if we run this test alone).
+  //  However, the second call should never block.
+  auto elapsed = end - start;
+  EXPECT_LT(std::chrono::duration_cast<std::chrono::milliseconds>
+      (elapsed).count(), 2);
+}
+
+//////////////////////////////////////////////////
+/// \brief Create a separate thread, block it calling waitForShutdown() and
+/// emit a SIGINT signal. Check that the transport library captures the signal
+/// and is able to terminate.
+TEST(NodeTest, waitForShutdownSIGINT)
+{
+  std::thread aThread([]{ignition::transport::waitForShutdown();});
+  std::this_thread::sleep_for(std::chrono::milliseconds(50));
+  raise(SIGINT);
+  aThread.join();
+}
+
+//////////////////////////////////////////////////
+/// \brief Create a separate thread, block it calling waitForShutdown() and
+/// emit a SIGTERM signal. Check that the transport library captures the signal
+/// and is able to terminate.TEST(NodeTest, TerminateSIGTERM)
+TEST(NodeTest, waitForShutdownSIGTERM)
+{
+  std::thread aThread([]{ignition::transport::waitForShutdown();});
+  std::this_thread::sleep_for(std::chrono::milliseconds(50));
+  raise(SIGTERM);
+  aThread.join();
+}
+
 //////////////////////////////////////////////////
 int main(int argc, char **argv)
 {
